import os
import textwrap
import platform
import pytest

from conan.tools.cmake import CMakeToolchain
from conan.tools.cmake.presets import load_cmake_presets
from conan.tools.files.files import load_toolchain_args
from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.genconanfile import GenConanfile
from conans.test.assets.sources import gen_function_h, gen_function_cpp
from conans.test.functional.utils import check_vs_runtime, check_exe_run
from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile
        from conan.tools.cmake import CMake, CMakeToolchain

        class Library(ConanFile):
            name = "hello"
            version = "1.0"
            settings = 'os', 'arch', 'compiler', 'build_type'
            exports_sources = 'hello.h', '*.cpp', 'CMakeLists.txt'
            options = {'shared': [True, False]}
            default_options = {'shared': False}

            def generate(self):
                tc = CMakeToolchain(self, generator="Ninja")
                tc.generate()

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
                self.run(os.sep.join([".", "myapp"]))

            def package(self):
                cmake = CMake(self)
                cmake.install()
        """)

    test_client = TestClient(path_with_spaces=False)
    test_client.save({'conanfile.py': conanfile,
                      "CMakeLists.txt": gen_cmakelists(libsources=["hello.cpp"],
                                                       appsources=["main.cpp"],
                                                       install=True),
                      "hello.h": gen_function_h(name="hello"),
                      "hello.cpp": gen_function_cpp(name="hello", includes=["hello"]),
                      "main.cpp": gen_function_cpp(name="main", includes=["hello"],
                                                   calls=["hello"])})
    return test_client


@pytest.mark.skipif(platform.system() != "Linux", reason="Only Linux")
@pytest.mark.parametrize("build_type,shared", [("Release", False), ("Debug", True)])
@pytest.mark.tool_ninja
def test_locally_build_linux(build_type, shared, client):
    settings = "-s os=Linux -s arch=x86_64 -s build_type={} -o hello:shared={}".format(build_type,
                                                                                       shared)
    client.run("install . {}".format(settings))
    client.run_command('cmake . -G "Ninja" -DCMAKE_TOOLCHAIN_FILE={} -DCMAKE_BUILD_TYPE={}'
                       .format(CMakeToolchain.filename, build_type))

    client.run_command('ninja')
    if shared:
        assert "Linking CXX shared library libmylibrary.so" in client.out
    else:
        assert "Linking CXX static library libmylibrary.a" in client.out

    client.run_command("./myapp")
    check_exe_run(client.out, ["main", "hello"], "gcc", None, build_type, "x86_64", cppstd=None)

    # create should also work
    client.run("create . hello/1.0@ {}".format(settings))
    assert 'cmake -G "Ninja"' in client.out
    assert "main: {}!".format(build_type) in client.out
    client.run("install hello/1.0@ -g=deploy -if=mydeploy {}".format(settings))
    ldpath = os.path.join(client.current_folder, "mydeploy", "hello", "lib")
    client.run_command("LD_LIBRARY_PATH='{}' ./mydeploy/hello/bin/myapp".format(ldpath))
    check_exe_run(client.out, ["main", "hello"], "gcc", None, build_type, "x86_64", cppstd=None)


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.parametrize("build_type,shared", [("Release", False), ("Debug", True)])
@pytest.mark.tool_ninja
def test_locally_build_msvc(build_type, shared, client):
    msvc_version = "15"
    settings = "-s build_type={} -o hello:shared={}".format(build_type, shared)
    client.run("install . {}".format(settings))

    client.run_command('conanvcvars.bat && cmake . -G "Ninja" '
                       '-DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake '
                       '-DCMAKE_BUILD_TYPE={}'.format(build_type))

    client.run_command("conanvcvars.bat && ninja")

    libname = "mylibrary.dll" if shared else "mylibrary.lib"
    assert libname in client.out

    client.run_command("myapp.exe")
    # TODO: Need full msvc version check
    check_exe_run(client.out, ["main", "hello"], "msvc", "19", build_type, "x86_64", cppstd="14")
    check_vs_runtime("myapp.exe", client, msvc_version, build_type, architecture="amd64")
    check_vs_runtime(libname, client, msvc_version, build_type, architecture="amd64")

    # create should also work
    client.run("create . hello/1.0@ {}".format(settings))
    assert 'cmake -G "Ninja"' in client.out
    assert "main: {}!".format(build_type) in client.out
    client.run("install hello/1.0@ -g=deploy -if=mydeploy {}".format(settings))
    client.run_command(r"mydeploy\hello\bin\myapp.exe")
    check_exe_run(client.out, ["main", "hello"], "msvc", "19", build_type, "x86_64", cppstd="14")


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.tool_ninja
def test_locally_build_msvc_toolset(client):
    msvc_version = "15"
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        compiler=msvc
        compiler.version=190
        compiler.runtime=dynamic
        compiler.cppstd=14
        build_type=Release
        arch=x86_64
        [conf]
        tools.cmake.cmaketoolchain:generator=Ninja
        tools.microsoft.msbuild:vs_version = 15
        """)
    client.save({"profile": profile})
    client.run("install . -pr=profile")

    client.run_command('conanvcvars.bat && cmake . -G "Ninja" '
                       '-DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake '
                       '-DCMAKE_BUILD_TYPE=Release')

    client.run_command("conanvcvars.bat && ninja")

    client.run_command("myapp.exe")

    # Checking that compiler is indeed version 19.0, not 19.1-default of VS15
    check_exe_run(client.out, ["main", "hello"], "msvc", "190", "Release", "x86_64", cppstd="14")
    check_vs_runtime("myapp.exe", client, msvc_version, "Release", architecture="amd64")
    check_vs_runtime("mylibrary.lib", client, msvc_version, "Release", architecture="amd64")


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.parametrize("build_type,shared", [("Release", False), ("Debug", True)])
@pytest.mark.tool_mingw64
@pytest.mark.tool_ninja
def test_locally_build_gcc(build_type, shared, client):
    # FIXME: Note the gcc version is still incorrect
    gcc = ("-s os=Windows -s compiler=gcc -s compiler.version=4.9 -s compiler.libcxx=libstdc++ "
           "-s arch=x86_64 -s build_type={}".format(build_type))

    client.run("install . {} -o hello:shared={}".format(gcc, shared))

    client.run_command('cmake . -G "Ninja" '
                       '-DCMAKE_TOOLCHAIN_FILE=conan_toolchain.cmake '
                       '-DCMAKE_BUILD_TYPE={}'.format(build_type))

    libname = "mylibrary.dll" if shared else "libmylibrary.a"
    client.run_command("ninja")
    assert libname in client.out

    client.run_command("myapp.exe")
    # TODO: Need full gcc version check
    check_exe_run(client.out, ["main", "hello"], "gcc", None, build_type, "x86_64", cppstd=None,
                  subsystem="mingw64")


@pytest.mark.skipif(platform.system() != "Darwin", reason="Requires apple-clang")
@pytest.mark.parametrize("build_type,shared", [("Release", False), ("Debug", True)])
@pytest.mark.tool_ninja
def test_locally_build_macos(build_type, shared, client):
    client.run('install . -s os=Macos -s arch=x86_64 -s build_type={} -o hello:shared={}'
               .format(build_type, shared))
    client.run_command('cmake . -G"Ninja" -DCMAKE_TOOLCHAIN_FILE={} -DCMAKE_BUILD_TYPE={}'
                       .format(CMakeToolchain.filename, build_type))

    client.run_command('ninja')
    if shared:
        assert "Linking CXX shared library libmylibrary.dylib" in client.out
    else:
        assert "Linking CXX static library libmylibrary.a" in client.out

    command_str = 'DYLD_LIBRARY_PATH="%s" ./myapp' % client.current_folder
    client.run_command(command_str)
    check_exe_run(client.out, ["main", "hello"], "apple-clang", None, build_type, "x86_64",
                  cppstd=None)


@pytest.mark.skipif(platform.system() != "Windows", reason="Only windows")
@pytest.mark.tool_visual_studio
def test_ninja_conf():
    conanfile = GenConanfile().with_generator("CMakeToolchain").with_settings("os", "compiler",
                                                                              "build_type", "arch")
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        compiler=msvc
        compiler.version=191
        compiler.runtime=dynamic
        compiler.cppstd=14
        build_type=Release
        arch=x86_64
        [conf]
        tools.cmake.cmaketoolchain:generator=Ninja
        """)
    client = TestClient()
    client.save({"conanfile.py": conanfile,
                 "profile": profile})
    client.run("install . -pr=profile")
    presets = load_cmake_presets(client.current_folder)
    generator = presets["configurePresets"][0]["generator"]

    assert generator == "Ninja"
    vcvars = client.load("conanvcvars.bat")
    assert "2017" in vcvars

    # toolchain cannot define the CMAKE_GENERATOR_TOOLSET for Ninja
    cmake = client.load("conan_toolchain.cmake")
    assert "CMAKE_GENERATOR_TOOLSET" not in cmake



import textwrap
import unittest

from conans.test.utils.tools import TestClient, GenConanfile


class RequireOverrideTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()

    def _save(self, req_method, requires):
        conanfile = GenConanfile()
        for req in requires:
            req2, override = req if isinstance(req, tuple) else (req, False)
            if not req_method:
                conanfile.with_require(req2, override=override)
            else:
                conanfile.with_requirement(req2, override=override)
        self.client.save({"conanfile.py": conanfile}, clean_first=True)

    def test_override(self):
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("export . libA/1.0@user/channel")
        # It is necessary to create libA/2.0 to have a conflict, otherwise it is missing
        self.client.run("export . libA/2.0@user/channel")

        for req_method in (False, True):
            self._save(req_method, ["libA/1.0@user/channel"])
            self.client.run("export . libB/1.0@user/channel")
            self._save(req_method, ["libA/2.0@user/channel"])
            self.client.run("export . libC/1.0@user/channel")
            self._save(req_method, ["libB/1.0@user/channel", "libC/1.0@user/channel"])
            self.client.run("info .", assert_error=True)
            self.assertIn("Conflict in libC/1.0@user/channel:\n"
                "    'libC/1.0@user/channel' requires 'libA/2.0@user/channel' while "
                "'libB/1.0@user/channel' requires 'libA/1.0@user/channel'.\n"
                "    To fix this conflict you need to override the package 'libA' in your root"
                " package.", self.client.out)

            self._save(req_method, ["libB/1.0@user/channel", "libC/1.0@user/channel",
                                    ("libA/1.0@user/channel", "override")])
            self.client.run("info .")
            self.assertIn("libA/2.0@user/channel overridden", self.client.out)

    def test_public_deps(self):
        client = TestClient()
        pkg2 = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                requires = ("pkg/0.1@user/stable", "override"),
                def package_info(self):
                    self.output.info("PUBLIC PKG2:%s" % self.cpp_info.public_deps)
            """)
        client.save({"conanfile.py": pkg2})
        client.run("create . pkg2/0.1@user/stable")
        self.assertIn("pkg2/0.1@user/stable: PUBLIC PKG2:[]", client.out)
        pkg3 = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                requires = "pkg2/0.1@user/stable", ("pkg/0.1@user/stable", "override")
                generators = "cmake"
            """)
        client.save({"conanfile.py": pkg3})
        client.run("install .")
        self.assertIn("pkg2/0.1@user/stable: PUBLIC PKG2:[]", client.out)
        conanbuildinfo = client.load("conanbuildinfo.cmake")
        self.assertIn("set(CONAN_DEPENDENCIES pkg2)", conanbuildinfo)



import platform
import textwrap
import unittest

import pytest

from conans.paths import CONANFILE
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient, TestServer

conanfile = textwrap.dedent('''
    from conans import ConanFile

    class ConanLib(ConanFile):
        name = "Hello0"
        version = "0.1"

        def source(self):
            self.output.info("Running source!")
''')


@pytest.mark.skipif(platform.system() == 'Linux', reason="Only for case insensitive OS")
class CaseSensitiveTest(unittest.TestCase):

    def test_install(self):
        test_server = TestServer()
        servers = {"default": test_server}
        client = TestClient(servers=servers, users={"default": [("lasote", "mypass")]})

        client.save({"conanfile.py": GenConanfile("Hello0", "0.1")})
        client.run("create . lasote/stable")
        client.run("upload Hello0/0.1@lasote/stable --all")

        client.save({"conanfile.py": GenConanfile().with_requires("hello0/0.1@lasote/stable")})
        client.run("install .", assert_error=True)
        self.assertIn("found case incompatible recipe with name 'Hello0' in the cache", client.out)

    def test_install_same(self):
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export . lasote/stable")
        client.run("install hello0/0.1@lasote/stable --build=missing", assert_error=True)
        self.assertIn("found case incompatible recipe with name 'Hello0' in the cache", client.out)

    def test_copy(self):
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export . lasote/stable")
        client.run("install Hello0/0.1@lasote/stable --build=missing")
        client.run("copy hello0/0.1@lasote/stable otheruser/testing", assert_error=True)
        self.assertIn("found case incompatible recipe with name 'Hello0' in the cache", client.out)

    def test_remove(self):
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export . lasote/stable")
        client.run("remove hello0/0.1@lasote/stable -f", assert_error=True)
        self.assertIn("found case incompatible recipe with name 'Hello0' in the cache", client.out)


class MismatchReference(unittest.TestCase):
    def test_imports(self):
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export . lasote/stable")
        client.run("install Hello0/0.1@lasote/stable --build=missing")
        client.run("imports hello0/0.1@lasote/stable", assert_error=True)
        # Reference interpreted as a path, so no valid path
        self.assertIn("Parameter 'path' cannot be a reference", client.out)

    def test_package(self):
        client = TestClient()
        client.save({CONANFILE: conanfile})
        client.run("export . lasote/stable")
        client.run("install Hello0/0.1@lasote/stable --build=missing")
        client.run("export-pkg . hello0/0.1@lasote/stable", assert_error=True)
        self.assertIn("ERROR: Package recipe with name hello0!=Hello0", client.out)



import textwrap

import pytest

from conan.tools.cmake.cmake import _cmake_cmd_line_args
from conans.model.conf import ConfDefinition
from conans.test.utils.mocks import ConanFileMock


@pytest.fixture
def conanfile():
    c = ConfDefinition()
    c.loads(textwrap.dedent("""\
        tools.build:jobs=10
    """))

    conanfile = ConanFileMock()
    conanfile.conf = c.get_conanfile_conf(None)
    return conanfile


def test_no_generator(conanfile):
    args = _cmake_cmd_line_args(conanfile, None)
    assert not len(args)


def test_makefiles(conanfile):
    args = _cmake_cmd_line_args(conanfile, 'Unix Makefiles')
    assert args == ['-j10']

    args = _cmake_cmd_line_args(conanfile, 'NMake Makefiles')
    assert not len(args)


def test_ninja(conanfile):
    args = _cmake_cmd_line_args(conanfile, 'Ninja')
    assert ['-j10'] == args


def test_visual_studio(conanfile):
    args = _cmake_cmd_line_args(conanfile, 'Visual Studio 16 2019')
    assert [] == args

    args = _cmake_cmd_line_args(conanfile, 'Ninja')
    assert args == ['-j10']



import pytest

from conan.tools.microsoft import check_min_vs
from conans.errors import ConanInvalidConfiguration
from conans.test.utils.mocks import MockConanfile, MockSettings


class TestCheckMinVS:

    @staticmethod
    def _create_conanfile(compiler, version, update=None):
        settings = MockSettings({"compiler": compiler,
                                 "compiler.version": version,
                                 "compiler.update": update})
        conanfile = MockConanfile(settings)
        return conanfile

    @pytest.mark.parametrize("compiler,version,update,minimum", [
        ("Visual Studio", "15", None, "191"),
        ("Visual Studio", "16", None, "192"),
        ("msvc", "193", None, "193"),
        ("msvc", "193", None, "192"),
        ("msvc", "193", "2", "193.1"),
    ])
    def test_valid(self, compiler, version, update, minimum):
        conanfile = self._create_conanfile(compiler, version, update)
        check_min_vs(conanfile, minimum)

    @pytest.mark.parametrize("compiler,version,update,minimum", [
        ("Visual Studio", "15", None, "192"),
        ("Visual Studio", "16", None, "193.1"),
        ("msvc", "192", None, "193"),
        ("msvc", "193", None, "193.1"),
        ("msvc", "193", "1", "193.2"),
    ])
    def test_invalid(self, compiler, version, update, minimum):
        conanfile = self._create_conanfile(compiler, version, update)
        with pytest.raises(ConanInvalidConfiguration):
            check_min_vs(conanfile, minimum)



from conans.client.subsystems import deduce_subsystem, subsystem_path


def unix_path(conanfile, path):
    if not conanfile.win_bash:
        return path
    subsystem = deduce_subsystem(conanfile, scope="build")
    return subsystem_path(subsystem, path)



# coding=utf-8

import os
import textwrap
import unittest
from jinja2 import Template

from conans.model.ref import ConanFileReference
from parameterized import parameterized
from conans.test.utils.tools import TestClient


class PythonRequireExportSourcesTest(unittest.TestCase):
    def setUp(self):
        self.pyreq = ConanFileReference.loads("py/req@user/channel")
        conanfile = textwrap.dedent("""\
            import os
            from conans import ConanFile, tools
            
            class PyReq(ConanFile):
                exports_sources = "exports_sources.txt", "more_exports_sources.txt"
                exports = "exports.txt", "more_exports.txt"
                
                def source(self, is_empty):
                    self.output.info(">>>> PyReq::source")
                    self.output.info(">>>> - source_folder: {{}}".format(self.source_folder))
                    file_list = sorted(os.listdir(self.source_folder))
                    self.output.info(">>>> - source list: {{}}".format(file_list))
                    if is_empty:
                        assert not file_list, "No files expected, found {{}}".format(file_list)
                    else:
                        assert file_list, "Files expected, but it is empty"
                        assert sorted(["more_exports_sources.txt", "more_exports.txt"]) == file_list
            """.format(pyreq_name=self.pyreq.name))

        self.t = TestClient()
        self.t.save({"conanfile.py": conanfile,
                     "exports_sources.txt": "exports_sources: {}".format(str(self.pyreq)),
                     "exports.txt": "exports: {}".format(str(self.pyreq))})
        self.t.run("export . {}".format(self.pyreq))
        package_layout = self.t.cache.package_layout(self.pyreq)
        self.assertListEqual(['exports_sources.txt'],
                             os.listdir(package_layout.export_sources()))
        self.assertListEqual(sorted(['exports.txt', 'conanfile.py', 'conanmanifest.txt']),
                             sorted(os.listdir(package_layout.export())))

    @parameterized.expand([(False, ), (True, )])
    def test_locate_pyreq_folders(self, empty_exports):
        conanfile = Template(textwrap.dedent("""\
            from conans import ConanFile, python_requires
            
            base = python_requires("{{ pyreq }}")
            
            class MyLib(base.PyReq):
                {% if empty %}
                exports_sources = None
                exports = None
                {% endif %}
                
                def source(self):
                    super(MyLib, self).source(is_empty={{ empty }})

                    self.output.info(">>>> MyLib::source")
                    pyreq = self.python_requires['{{ pyreq.name }}']
                    self.output.info(" - pyreq::ref: {{{}}}".format(str(pyreq.ref)))
                    self.output.info(" - pyreq::exports_sources: {{{}}}".format(pyreq.exports_sources_folder))
                    self.output.info(" - pyreq::exports: {{{}}}".format(pyreq.exports_folder))
            
            """))
        content = conanfile.render(pyreq=self.pyreq, empty=empty_exports)

        ref = ConanFileReference.loads("lib/version@user/channel")
        self.t.save({"conanfile.py": content}, clean_first=True)
        if not empty_exports:
            # Write some files, these will be exported because they match patterns in pyreq
            self.t.save({"more_exports.txt": "...",
                         "more_exports_sources.txt": "..."})
        self.t.run("create . {}".format(ref))

        # Check things written in 'source' method (source folder executed in consumer context)
        source_folder = self.t.cache.package_layout(ref).source()
        self.assertIn(">>>> PyReq::source", self.t.out)
        self.assertIn(">>>> - source_folder: {}".format(source_folder), self.t.out)
        if not empty_exports:
            self.assertIn(">>>> - source list: ['more_exports.txt', 'more_exports_sources.txt']",
                          self.t.out)
        else:
            self.assertIn(">>>> - source list: []", self.t.out)

        # Access to sources from pyreq, check folders match
        pyreq_layout = self.t.cache.package_layout(self.pyreq)
        self.assertIn(" - pyreq::ref: {}".format(self.pyreq), self.t.out)
        self.assertIn(" - pyreq::exports_sources: %s" % pyreq_layout.export_sources(), self.t.out)
        self.assertIn(" - pyreq::exports: %s" % pyreq_layout.export(), self.t.out)



import textwrap
import unittest

from conans.client.generators.text import TXTGenerator


class LoadContentTestCase(unittest.TestCase):

    def test_load_sytem_libs(self):
        # Loading an item that contains an underscore
        content = textwrap.dedent("""
            [system_libs]
            a-real-flow-contains-aggregated-list-here

            [name_requirement]
            requirement_name

            [rootpath_requirement]
            requirement_rootpath

            [system_libs_requirement]
            requirement

            [name_requirement_other]
            requirement_other_name

            [rootpath_requirement_other]
            requirement_other_rootpath

            [system_libs_requirement_other]
            requirement_other
        """)

        deps_cpp_info, _, _, _ = TXTGenerator.loads(content)
        self.assertListEqual(list(deps_cpp_info.system_libs), ["requirement", "requirement_other"])
        self.assertListEqual(list(deps_cpp_info["requirement"].system_libs), ["requirement", ])
        self.assertListEqual(list(deps_cpp_info["requirement_other"].system_libs),
                             ["requirement_other", ])



import tempfile
import unittest
from collections import namedtuple

import six

from conans.client.rest.file_uploader import FileUploader
from conans.errors import AuthenticationException, ForbiddenException, InternalErrorException
from conans.test.utils.mocks import TestBufferConanOutput
from conans.util.files import save


class _ConfigMock:
    def __init__(self):
        self.retry = 0
        self.retry_wait = 0


class MockRequester(object):
    retry = 0
    retry_wait = 0

    def __init__(self, response):
        self._response = response

    def put(self, *args, **kwargs):
        return namedtuple("response", "status_code content")(self._response, "tururu")


class UploaderUnitTest(unittest.TestCase):
    def setUp(self):
        _, self.f = tempfile.mkstemp()
        save(self.f, "some contents")
        self.out = TestBufferConanOutput()

    def test_401_raises_unauthoirzed_exception(self):
        uploader = FileUploader(MockRequester(401), self.out, verify=False, config=_ConfigMock())
        with six.assertRaisesRegex(self, AuthenticationException, "tururu"):
            uploader.upload("fake_url", self.f)

    def test_403_raises_unauthoirzed_exception_if_no_token(self):
        auth = namedtuple("auth", "token")(None)
        uploader = FileUploader(MockRequester(403), self.out, verify=False, config=_ConfigMock())
        with six.assertRaisesRegex(self, AuthenticationException, "tururu"):
            uploader.upload("fake_url", self.f, auth=auth)

    def test_403_raises_unauthorized_exception_if_no_auth(self):
        uploader = FileUploader(MockRequester(403), self.out, verify=False, config=_ConfigMock())
        with six.assertRaisesRegex(self, AuthenticationException, "tururu"):
            uploader.upload("fake_url", self.f)

    def test_403_raises_forbidden_exception_if_token(self):
        auth = namedtuple("auth", "token")("SOMETOKEN")
        uploader = FileUploader(MockRequester(403), self.out, verify=False, config=_ConfigMock())
        with six.assertRaisesRegex(self, ForbiddenException, "tururu"):
            uploader.upload("fake_url", self.f, auth=auth)

    def test_500_raises_internal_error(self):
        out = TestBufferConanOutput()
        uploader = FileUploader(MockRequester(500), out, verify=False, config=_ConfigMock())
        _, f = tempfile.mkstemp()
        save(f, "some contents")
        with six.assertRaisesRegex(self, InternalErrorException, "tururu"):
            uploader.upload("fake_url", self.f, dedup=True)



import os
import unittest

# Check it is importable from tools
from conans.tools import remove_files_by_mask
from conans.client.tools.files import chdir
from conans.test.utils.test_files import temp_folder
from conans.util.files import save_files


class RemoveFilesByMaskTest(unittest.TestCase):
    def test_remove_files_by_mask(self):
        tmpdir = temp_folder()

        with chdir(tmpdir):
            os.makedirs("subdir")
            os.makedirs("dir.pdb")
            os.makedirs(os.path.join("subdir", "deepdir"))

        save_files(tmpdir, {"1.txt": "",
                            "1.pdb": "",
                            "1.pdb1": "",
                            os.path.join("subdir", "2.txt"): "",
                            os.path.join("subdir", "2.pdb"): "",
                            os.path.join("subdir", "2.pdb1"): "",
                            os.path.join("subdir", "deepdir", "3.txt"): "",
                            os.path.join("subdir", "deepdir", "3.pdb"): "",
                            os.path.join("subdir", "deepdir", "3.pdb1"): ""})

        removed_files = remove_files_by_mask(tmpdir, "*.sh")
        self.assertEqual(removed_files, [])

        removed_files = remove_files_by_mask(tmpdir, "*.pdb")

        self.assertTrue(os.path.isdir(os.path.join(tmpdir, "dir.pdb")))

        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "1.txt")))
        self.assertFalse(os.path.isfile(os.path.join(tmpdir, "1.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "1.pdb1")))

        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "subdir", "2.txt")))
        self.assertFalse(os.path.isfile(os.path.join(tmpdir, "subdir", "2.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "subdir", "2.pdb1")))

        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "subdir", "deepdir", "3.txt")))
        self.assertFalse(os.path.isfile(os.path.join(tmpdir, "subdir", "deepdir", "3.pdb")))
        self.assertTrue(os.path.isfile(os.path.join(tmpdir, "subdir", "deepdir", "3.pdb1")))

        self.assertEqual(set(removed_files), {"1.pdb",
                                              os.path.join("subdir", "2.pdb"),
                                              os.path.join("subdir", "deepdir", "3.pdb")})

        removed_files = remove_files_by_mask(tmpdir, "*.pdb")
        self.assertEqual(removed_files, [])



import platform
import sys
import textwrap
import os

import pytest
try:
    from unittest.mock import MagicMock
except:
    from mock import MagicMock

from conan.tools.files import replace_in_file
from conans.test.utils.tools import TestClient

toolchain_props = """
  <ImportGroup Label="PropertySheets">
      <Import Project="conan\\conantoolchain_release_x64.props" />
"""


@pytest.mark.skipif(sys.version_info.major == 2, reason="Meson not supported in Py2")
@pytest.mark.skipif(platform.system() not in ["Windows"], reason="Requires Windows")
def test_msbuildtoolchain_props_with_extra_flags():
    """
    Real test which is injecting some compiler/linker options and other dummy defines and
    checking that they are being processed somehow.

    Expected result: everything was built successfully.
    """
    profile = textwrap.dedent("""\
    include(default)

    [conf]
    tools.build:cxxflags=["/analyze:quiet"]
    tools.build:cflags+=["/doc"]
    tools.build:sharedlinkflags+=["/VERBOSE:UNUSEDLIBS"]
    tools.build:exelinkflags+=["/PDB:mypdbfile"]
    tools.build:defines+=["DEF1", "DEF2"]
    """)
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=msbuild_exe")
    client.save({
        "myprofile": profile
    })
    # Let's import manually the created conantoolchain_release_x64.props
    replace_in_file(MagicMock(), os.path.join(client.current_folder, "hello.vcxproj"),
                    r'  <ImportGroup Label="PropertySheets">', toolchain_props)
    client.run("create . -pr myprofile -tf None")
    assert "/analyze:quiet /doc src/hello.cpp" in client.out
    assert r"/VERBOSE:UNUSEDLIBS /PDB:mypdbfile x64\Release\hello.obj" in client.out
    assert "/D DEF1 /D DEF2" in client.out
    assert "Build succeeded." in client.out



import os
import platform
import textwrap
import unittest

from conans.client.tools.intel import intel_installation_path
from conans.errors import ConanException
from conans.test.utils.tools import TestClient


class BaseIntelTestCase(unittest.TestCase):
    def setUp(self):
        try:
            installation_path = intel_installation_path(self.version, self.arch)
            if not os.path.isdir(installation_path):
                raise unittest.SkipTest("Intel C++ compiler is required")
        except ConanException:
            raise unittest.SkipTest("Intel C++ compiler is required")

        self.t = TestClient()

    @property
    def version(self):
        return '19.1'

    @property
    def arch(self):
        return "x86_64"

    @property
    def settings(self):
        the_os = {'Darwin': 'Macos'}.get(platform.system(), platform.system())
        vars = [('compiler', 'intel'),
                ('compiler.version', self.version),
                ('build_type', 'Release'),
                ('arch', self.arch),
                ('os', the_os)]
        if platform.system() == "Windows":
            vars.append(('compiler.base', 'Visual Studio'))
            vars.append(('compiler.base.version', '15'))
            vars.append(('compiler.base.runtime', 'MD'))
        else:
            vars.append(('compiler.base', 'gcc'))
            vars.append(('compiler.base.version', '10'))
            vars.append(('compiler.base.libcxx', 'libstdc++'))
        return vars

    @property
    def profile(self):
        template = textwrap.dedent("""
            include(default)
            [settings]
            {settings}
            """)
        settings = '\n'.join(["%s = %s" % (s[0], s[1]) for s in self.settings])
        return template.format(settings=settings)



# coding=utf-8

import unittest
from collections import namedtuple

import six

from conans.errors import ConanException
from conans.model.scm import _get_dict_value, SCMData


class GetDictValueTestCase(unittest.TestCase):

    def test_string_type(self):
        self.assertEqual(_get_dict_value({"str": "value"}, "str", six.string_types), "value")
        self.assertEqual(_get_dict_value({"str": u"value"}, "str", six.string_types), "value")

        def get_string():
            return "value"
        self.assertEqual(_get_dict_value({"str": get_string()}, "str", six.string_types), "value")
        self.assertEqual(_get_dict_value({"str": None}, "str", six.string_types), None)

    def test_no_string_type(self):
        str_type_name = 'str' if six.PY3 else 'basestring'
        exception_msg = "must be of type '{}' \(found '{{found}}'\)".format(str_type_name)
        with six.assertRaisesRegex(self, ConanException, exception_msg.format(found="int")):
            _get_dict_value({"str": 23}, "str", six.string_types)

        if six.PY3:  # For py2, bytes is instance of str
            with six.assertRaisesRegex(self, ConanException, exception_msg.format(found="bytes")):
                _get_dict_value({"str": b"value"}, "str", six.string_types)

        with six.assertRaisesRegex(self, ConanException, exception_msg.format(found="bool")):
            _get_dict_value({"str": True}, "str", six.string_types)

        with six.assertRaisesRegex(self, ConanException, exception_msg.format(found="function")):
            _get_dict_value({"str": lambda: "value"}, "str", six.string_types)

    def test_boolean_type(self):
        self.assertEqual(_get_dict_value({"key": True}, "key", bool), True)
        self.assertEqual(_get_dict_value({"key": False}, "key", bool), False)

    def test_no_boolean_type(self):
        with six.assertRaisesRegex(self, ConanException, "must be of type 'bool' \(found 'int'\)"):
            _get_dict_value({"key": 123}, "key", bool)

        with six.assertRaisesRegex(self, ConanException, "must be of type 'bool' \(found 'str'\)"):
            _get_dict_value({"key": "str"}, "key", bool)


class SCMDataToStringTestCase(unittest.TestCase):
    data = {"url": "http://my.url",
            "revision": 123,
            "shallow": False,
            "username": 'weir"d',
            "type": "weir\"d",
            "password": "don't"}

    def test_scmdata_string(self):
        fake_class = namedtuple("Conanfile", ["scm"])
        conanfile = fake_class(scm=self.data)

        self.assertEqual(str(SCMData(conanfile)), '{"password": "don\'t", "revision": "123",'
                                                  ' "shallow": False, "type": "weir\\"d",'
                                                  ' "url": "http://my.url", "username": "weir\\"d"}')



import os
import textwrap

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_package_from_system():
    """
    If a node declares "system_package" property, the CMakeDeps generator will skip generating
    the -config.cmake and the other files for that node but will keep the "find_dependency" for
    the nodes depending on it. That will cause that cmake looks for the config files elsewhere
    https://github.com/conan-io/conan/issues/8919"""
    client = TestClient()
    dep2 = str(GenConanfile().with_name("dep2").with_version("1.0")
               .with_settings("os", "arch", "build_type", "compiler"))
    dep2 += """
    def package_info(self):
        self.cpp_info.set_property("cmake_find_mode", "None")
        self.cpp_info.set_property("cmake_file_name", "custom_dep2")

    """
    client.save({"conanfile.py": dep2})
    client.run("create .")

    dep1 = GenConanfile().with_name("dep1").with_version("1.0").with_require("dep2/1.0")\
                         .with_settings("os", "arch", "build_type", "compiler")
    client.save({"conanfile.py": dep1})
    client.run("create .")

    consumer = GenConanfile().with_name("consumer").with_version("1.0").\
        with_require("dep1/1.0").with_generator("CMakeDeps").\
        with_settings("os", "arch", "build_type", "compiler")
    client.save({"conanfile.py": consumer})
    client.run("install .")

    assert os.path.exists(os.path.join(client.current_folder, "dep1-config.cmake"))
    assert not os.path.exists(os.path.join(client.current_folder, "dep2-config.cmake"))
    assert not os.path.exists(os.path.join(client.current_folder, "custom_dep2-config.cmake"))
    host_arch = client.get_default_host_profile().settings['arch']
    dep1_contents = client.load(f"dep1-release-{host_arch}-data.cmake")
    assert 'list(APPEND dep1_FIND_DEPENDENCY_NAMES custom_dep2)' in dep1_contents
    assert 'set(custom_dep2_FIND_MODE "")' in dep1_contents


def test_test_package():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . gtest/1.0@")
    client.run("create . cmake/1.0@")

    client.save({"conanfile.py": GenConanfile().with_build_requires("cmake/1.0").
                with_build_requirement("gtest/1.0", force_host_context=True)})

    client.run("export . pkg/1.0@")

    consumer = textwrap.dedent(r"""
        from conans import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            generators = "CMakeDeps"
            requires = "pkg/1.0"
        """)
    client.save({"conanfile.py": consumer})
    client.run("install . -s:b os=Windows -s:h os=Linux -s:h arch=x86_64 --build=missing")
    cmake_data = client.load("pkg-release-x86_64-data.cmake")
    assert "gtest" not in cmake_data


def test_components_error():
    # https://github.com/conan-io/conan/issues/9331
    client = TestClient()

    conan_hello = textwrap.dedent("""
        import os
        from conans import ConanFile

        from conan.tools.files import save
        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"

            def layout(self):
                pass

            def package_info(self):
                self.cpp_info.components["say"].includedirs = ["include"]
            """)

    client.save({"conanfile.py": conan_hello})
    client.run("create . hello/1.0@")


def test_cpp_info_component_objects():
    client = TestClient()
    conan_hello = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            settings = "os", "arch", "build_type"
            def package_info(self):
                self.cpp_info.components["say"].objects = ["mycomponent.o"]
            """)

    client.save({"conanfile.py": conan_hello})
    client.run("create . hello/1.0@ -s arch=x86_64 -s build_type=Release")
    client.run("install hello/1.0@ -g CMakeDeps -s arch=x86_64 -s build_type=Release")
    with open(os.path.join(client.current_folder, "hello-Target-release.cmake")) as f:
        content = f.read()
        assert """set_property(TARGET hello::say
                     PROPERTY INTERFACE_LINK_LIBRARIES
                     $<$<CONFIG:Release>:${hello_hello_say_OBJECTS_RELEASE}>
                     $<$<CONFIG:Release>:${hello_hello_say_LIBRARIES_TARGETS}>
                     APPEND)""" in content
        # If there are componets, there is not a global cpp so this is not generated
        assert "hello_OBJECTS_RELEASE" not in content
        # But the global target is linked with the targets from the components
        assert "set_property(TARGET hello::hello PROPERTY INTERFACE_LINK_LIBRARIES " \
               "hello::say APPEND)" in content

    with open(os.path.join(client.current_folder, "hello-release-x86_64-data.cmake")) as f:
        content = f.read()
        # https://github.com/conan-io/conan/issues/11862
        # Global variables
        assert 'set(hello_OBJECTS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/mycomponent.o")' \
               in content
        # But component variables
        assert 'set(hello_hello_say_OBJECTS_RELEASE "${hello_PACKAGE_FOLDER_RELEASE}/' \
               'mycomponent.o")' in content


def test_cpp_info_component_error_aggregate():
    # https://github.com/conan-io/conan/issues/10176
    # This test was consistently failing because "VirtualRunEnv" was not doing a "copy()"
    # of cpp_info before calling "aggregate_components()", and it was destructive, removing
    # components data
    client = TestClient()
    conan_hello = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def package_info(self):
                self.cpp_info.components["say"].includedirs = ["include"]
            """)
    consumer = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            requires = "hello/1.0"
            generators = "VirtualRunEnv", "CMakeDeps"
            def package_info(self):
                self.cpp_info.components["chat"].requires = ["hello::say"]
        """)
    test_package = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "arch", "build_type"
            generators = "VirtualRunEnv", "CMakeDeps"

            def test(self):
                pass
        """)

    client.save({"hello/conanfile.py": conan_hello,
                 "consumer/conanfile.py": consumer,
                 "consumer/test_package/conanfile.py": test_package})
    client.run("create hello hello/1.0@")
    client.run("create consumer consumer/1.0@")
    assert "consumer/1.0 (test package): Running test()" in client.out


def test_cmakedeps_cppinfo_complex_strings():
    client = TestClient(path_with_spaces=False)
    conanfile = textwrap.dedent(r'''
        from conans import ConanFile
        class HelloLib(ConanFile):
            def package_info(self):
                self.cpp_info.defines.append("escape=partially \"escaped\"")
                self.cpp_info.defines.append("spaces=me you")
                self.cpp_info.defines.append("foobar=bazbuz")
                self.cpp_info.defines.append("answer=42")
        ''')
    client.save({"conanfile.py": conanfile})
    client.run("export . hello/1.0@")
    client.save({"conanfile.txt": "[requires]\nhello/1.0\n"}, clean_first=True)
    client.run("install . --build=missing -g CMakeDeps")
    arch = client.get_default_host_profile().settings['arch']
    deps = client.load(f"hello-release-{arch}-data.cmake")
    assert r"escape=partially \"escaped\"" in deps
    assert r"spaces=me you" in deps
    assert r"foobar=bazbuz" in deps
    assert r"answer=42" in deps



import json
import textwrap
import unittest

from conans.model.graph_lock import LOCKFILE
from conans.test.utils.tools import TestClient, GenConanfile
from conans.util.env_reader import get_env


class GraphLockPyRequiresTransitiveTest(unittest.TestCase):
    def test_transitive_py_requires(self):
        # https://github.com/conan-io/conan/issues/5529
        client = TestClient()
        client.save({"conanfile.py": GenConanfile()})
        client.run("export . base/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class PackageInfo(ConanFile):
                python_requires = "base/1.0@user/channel"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . helper/1.0@user/channel")

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class MyConanfileBase(ConanFile):
                python_requires = "helper/1.0@user/channel"
            """)
        client.save({"conanfile.py": conanfile})
        client.run("install . pkg/0.1@user/channel")
        lockfile = client.load("conan.lock")
        if client.cache.config.revisions_enabled:
            self.assertIn("base/1.0@user/channel#f3367e0e7d170aa12abccb175fee5f97", lockfile)
            self.assertIn("helper/1.0@user/channel#539219485c7a9e8e19561db523512b39", lockfile)
        else:
            self.assertIn("base/1.0@user/channel", lockfile)
            self.assertIn("helper/1.0@user/channel", lockfile)

        client.run("source .")
        self.assertIn("conanfile.py (pkg/0.1@user/channel): Configuring sources in", client.out)

    def test_inherit_with_init(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            import os, re
            class Base(object):
                def init(self):
                    self.versionFile = "1.2.3"
                def set_name(self):
                    self.name = self.bundleName
                def set_version(self):
                    self.version = self.versionFile

            class PythonRequires(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . pyreq/1.3.0@")
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class BundleBuilder(ConanFile):
                python_requires = "pyreq/1.3.0"
                python_requires_extend = "pyreq.Base"
                bundleName = "BASE"
            """)
        client.save({"conanfile.py": conanfile}, clean_first=True)

        client.run("lock create conanfile.py --base --lockfile-out=base.lock")
        base_lock = client.load("base.lock")
        self.assertIn('pyreq/1.3.0', base_lock)

        client.run("lock create conanfile.py --lockfile=base.lock --lockfile-out=conan.lock")
        conan_lock = client.load("conan.lock")
        self.assertIn('pyreq/1.3.0', conan_lock)


class GraphLockPyRequiresTest(unittest.TestCase):
    if get_env("TESTING_REVISIONS_ENABLED", False):
        pkg_ref = "Pkg/0.1@user/channel#67fdc942d6157fc4db1971fd6d6c5c28"
    else:
        pkg_ref = "Pkg/0.1@user/channel"
    pkg_id = "1e1576940da80e70cd2d2ce2dddeb0571f91c6e3"
    consumer = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            name = "Pkg"
            python_requires = "Tool/[>=0.1]@user/channel"
            def configure(self):
                self.output.info("CONFIGURE VAR=%s" % self.python_requires["Tool"].module.var)
            def build(self):
                self.output.info("BUILD VAR=%s" % self.python_requires["Tool"].module.var)
        """)

    def setUp(self):
        client = TestClient()
        self.client = client
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            var = 42
            class Pkg(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("export . Tool/0.1@user/channel")

        # Use a consumer with a version range
        client.save({"conanfile.py": self.consumer})
        client.run("install . Pkg/0.1@user/channel")
        self.assertIn("Tool/0.1@user/channel", client.out)
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): CONFIGURE VAR=42", client.out)
        self._check_lock("Pkg/0.1@user/channel")

        client.run("build .")
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): CONFIGURE VAR=42", client.out)
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): BUILD VAR=42", client.out)

        # If we create a new Tool version
        client.save({"conanfile.py": conanfile.replace("42", "111")})
        client.run("export . Tool/0.2@user/channel")
        client.save({"conanfile.py": self.consumer})

    def _check_lock(self, ref_b, pkg_id_b=None):
        lock_file = self.client.load(LOCKFILE)
        self.assertIn("Tool/0.1@user/channel", lock_file)
        self.assertNotIn("Tool/0.2@user/channel", lock_file)
        lock_file_json = json.loads(lock_file)
        nodes = lock_file_json["graph_lock"]["nodes"]
        self.assertEqual(1, len(nodes))
        pkg = nodes["0"]
        self.assertEqual(pkg["ref"], ref_b)
        if self.client.cache.config.revisions_enabled:
            tool = "Tool/0.1@user/channel#ac4036130c39cab7715b1402e8c211d3"
        else:
            tool = "Tool/0.1@user/channel"
        self.assertEqual(pkg["python_requires"], [tool])
        self.assertEqual(pkg.get("package_id"), pkg_id_b)

    def test_install_info(self):
        client = self.client
        # Make sure to use temporary if to not change graph_info.json
        client.run("install . -if=tmp")
        self.assertIn("Tool/0.2@user/channel", client.out)
        client.run("build . -if=tmp")
        self.assertIn("conanfile.py (Pkg/None): CONFIGURE VAR=111", client.out)
        self.assertIn("conanfile.py (Pkg/None): BUILD VAR=111", client.out)

        client.run("install . --lockfile=conan.lock")
        self.assertIn("Tool/0.1@user/channel", client.out)
        self.assertNotIn("Tool/0.2@user/channel", client.out)
        self._check_lock("Pkg/0.1@user/channel")
        client.run("build .")
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): CONFIGURE VAR=42", client.out)
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): BUILD VAR=42", client.out)

        client.run("package .")
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): CONFIGURE VAR=42", client.out)

        client.run("info . --lockfile=conan.lock")
        self.assertIn("conanfile.py (Pkg/0.1@user/channel): CONFIGURE VAR=42", client.out)

    def test_create(self):
        client = self.client
        client.run("create . Pkg/0.1@user/channel --lockfile=conan.lock --lockfile-out=conan.lock")
        self.assertIn("Pkg/0.1@user/channel: CONFIGURE VAR=42", client.out)
        self.assertIn("Pkg/0.1@user/channel: BUILD VAR=42", client.out)
        self.assertIn("Tool/0.1@user/channel", client.out)
        self.assertNotIn("Tool/0.2@user/channel", client.out)
        self._check_lock(self.pkg_ref, self.pkg_id)

    def test_export_pkg(self):
        client = self.client
        client.run("export-pkg . Pkg/0.1@user/channel --install-folder=. --lockfile=conan.lock "
                   "--lockfile-out=conan.lock")
        self.assertIn("Pkg/0.1@user/channel: CONFIGURE VAR=42", client.out)
        self._check_lock(self.pkg_ref, self.pkg_id)


class GraphLockPythonRequiresTest(GraphLockPyRequiresTest):
    if get_env("TESTING_REVISIONS_ENABLED", False):
        pkg_ref = "Pkg/0.1@user/channel#332c2615c2ff9f78fc40682e733e5aa5"
    else:
        pkg_ref = "Pkg/0.1@user/channel"
    pkg_id = "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
    consumer = textwrap.dedent("""
       from conans import ConanFile, python_requires
       dep = python_requires("Tool/[>=0.1]@user/channel")

       class Pkg(ConanFile):
           name = "Pkg"
           def configure(self):
               self.output.info("CONFIGURE VAR=%s" % dep.var)
           def build(self):
               self.output.info("BUILD VAR=%s" % dep.var)
       """)



import uuid

import pytest

from conans.util import encrypt


def test_encryp_basic():
    key = str(uuid.uuid4())
    message = 'simple data ascii string'

    data = encrypt.encode(message, key)
    assert type(message) == type(data)
    assert message != data
    assert message != data

    decoded = encrypt.decode(data, key)
    assert type(message) == type(data)
    assert message == decoded


def test_encrypt_unicode():
    key = str(uuid.uuid4())
    message_enc = b'espa\xc3\xb1a\xe2\x82\xac$'  # Conan codebase allows only ASCII source files
    message = message_enc.decode('utf-8')

    data = encrypt.encode(message, key)
    assert type(message) == type(data)
    assert message != data

    decoded = encrypt.decode(data, key)
    assert type(message) == type(data)
    assert message == decoded


def test_key_unicode():
    key = b'espa\xc3\xb1a\xe2\x82\xac$'.decode('utf-8')  # codebase allows only ASCII files
    message = 'the message'

    data = encrypt.encode(message, key)
    assert type(message) == type(data)
    assert message != data

    decoded = encrypt.decode(data, key)
    assert type(message) == type(data)
    assert message == decoded


def test_key_empty():
    # Empty keys, or keys with only non-ascii chars are not allowed
    with pytest.raises(AssertionError):
        encrypt.encode('message', '')

    with pytest.raises(AssertionError):
        encrypt.encode('message', b'\xc3\xb1\xe2\x82\xac'.decode('utf-8'))



import os
import platform
import shutil
import unittest

import pytest


from conans.test.assets.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import TestClient, TestServer
from conans.util.files import rmdir


@pytest.mark.slow
class SharedChainTest(unittest.TestCase):

    def setUp(self):
        self.servers = {"default": TestServer()}

    def _export_upload(self, name, version=None, deps=None):
        conan = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        files = cpp_hello_conan_files(name, version, deps, static=False)
        conan.save(files)

        conan.run("create . lasote/stable")
        conan.run("upload * --all --confirm")
        conan.run("remove * -f")
        rmdir(conan.current_folder)
        shutil.rmtree(conan.cache.store, ignore_errors=True)

    def test_uploaded_chain(self):
        self._export_upload("Hello0", "0.1")
        self._export_upload("Hello1", "0.1", ["Hello0/0.1@lasote/stable"])

        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        files = cpp_hello_conan_files("Hello2", "0.1", ["Hello1/0.1@lasote/stable"], static=True)
        c = files["conanfile.py"]
        c = c.replace("def imports(self):", "def imports(self):\n"
                                            '        self.copy(pattern="*.so", dst=".", src="lib")')
        files["conanfile.py"] = c
        client.save(files)

        client.run("install .")
        client.run("build .")
        ld_path = (
            "LD_LIBRARY_PATH='{}' ".format(client.current_folder)
            if platform.system() != "Windows"
            else ""
        )
        cmd_path = os.sep.join([".", "bin", "say_hello"])
        command = ld_path + cmd_path

        client.run_command(command)
        self.assertEqual(['Hello Hello2', 'Hello Hello1', 'Hello Hello0'],
                         str(client.out).splitlines()[-3:])



import pytest

from conan.tools.build import supported_cppstd, check_min_cppstd, valid_min_cppstd
from conans.errors import ConanException, ConanInvalidConfiguration
from conans.test.utils.mocks import MockSettings, MockConanfile


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("clang", "2.0", []),
    ("clang", "2.1", ['98', 'gnu98', '11', 'gnu11']),
    ("clang", "2.2", ['98', 'gnu98', '11', 'gnu11']),
    ("clang", "3.1", ['98', 'gnu98', '11', 'gnu11']),
    ("clang", "3.4", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14"]),
    ("clang", "3.5", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("clang", "4.9", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("clang", "5", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("clang", "6", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20", "gnu20"]),
    ("clang", "12", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20",
                     "gnu20", "23", "gnu23"])
])
def test_supported_cppstd_clang(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile)
    assert sot == values


def test_supported_cppstd_with_specific_values():
    settings = MockSettings({})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile, "clang", "3.1")
    assert sot == ['98', 'gnu98', '11', 'gnu11']


def test_supported_cppstd_error():
    settings = MockSettings({})
    conanfile = MockConanfile(settings)
    with pytest.raises(ConanException) as exc:
        supported_cppstd(conanfile)
    assert "Called supported_cppstd with no compiler or no compiler.version" in str(exc)


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("gcc", "2.0", []),
    ("gcc", "3.4", ['98', 'gnu98']),
    ("gcc", "4.2", ['98', 'gnu98']),
    ("gcc", "4.3", ['98', 'gnu98', '11', 'gnu11']),
    ("gcc", "4.8", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14"]),
    ("gcc", "5", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("gcc", "8", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20", "gnu20"]),
    ("gcc", "11", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20", "gnu20",
                   "23", "gnu23"])
])
def test_supported_cppstd_gcc(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile)
    assert sot == values


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("apple-clang", "3.9", []),
    ("apple-clang", "4.0", ['98', 'gnu98', '11', 'gnu11']),
    ("apple-clang", "5.0", ['98', 'gnu98', '11', 'gnu11']),
    ("apple-clang", "5.1", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14"]),
    ("apple-clang", "6.1", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("apple-clang", "9.5", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("apple-clang", "10", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20",
                           "gnu20"]),
])
def test_supported_cppstd_apple_clang(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile)
    assert sot == values


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("msvc", "180", []),
    ("msvc", "190", ['14', '17']),
    ("msvc", "191", ['14', '17', '20']),
    ("msvc", "193", ['14', '17', '20', '23']),
])
def test_supported_cppstd_msvc(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile)
    assert sot == values


@pytest.mark.parametrize("compiler,compiler_version,values", [
    ("mcst-lcc", "1.20", ['98', 'gnu98']),
    ("mcst-lcc", "1.21", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14"]),
    ("mcst-lcc", "1.23", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14"]),
    ("mcst-lcc", "1.24", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17"]),
    ("mcst-lcc", "1.25", ['98', 'gnu98', '11', 'gnu11', "14", "gnu14", "17", "gnu17", "20", "gnu20"])
])
def test_supported_cppstd_mcst(compiler, compiler_version, values):
    settings = MockSettings({"compiler": compiler, "compiler.version": compiler_version})
    conanfile = MockConanfile(settings)
    sot = supported_cppstd(conanfile)
    assert sot == values


def test_check_cppstd_type():
    """ cppstd must be a number
    """
    conanfile = MockConanfile(MockSettings({}))
    with pytest.raises(ConanException) as exc:
        check_min_cppstd(conanfile, "gnu17", False)

    assert "cppstd parameter must be a number", str(exc)


def _create_conanfile(compiler, version, os, cppstd, libcxx=None):
    settings = MockSettings({"arch": "x86_64",
                             "build_type": "Debug",
                             "os": os,
                             "compiler": compiler,
                             "compiler.version": version,
                             "compiler.cppstd": cppstd})
    if libcxx:
        settings.values["compiler.libcxx"] = libcxx
    conanfile = MockConanfile(settings)
    return conanfile


@pytest.mark.parametrize("cppstd", ["98", "11", "14", "17"])
def test_check_min_cppstd_from_settings(cppstd):
    """ check_min_cppstd must accept cppstd less/equal than cppstd in settings
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", "17", "libstdc++")
    check_min_cppstd(conanfile, cppstd, False)


@pytest.mark.parametrize("cppstd", ["98", "11", "14"])
def test_check_min_cppstd_from_outdated_settings(cppstd):
    """ check_min_cppstd must raise when cppstd is greater when supported on settings
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", cppstd, "libstdc++")
    with pytest.raises(ConanInvalidConfiguration) as exc:
        check_min_cppstd(conanfile, "17", False)
    assert "Current cppstd ({}) is lower than the required C++ standard (17)." \
           "".format(cppstd) == str(exc.value)


@pytest.mark.parametrize("cppstd", ["98", "11", "14", "17"])
def test_check_min_cppstd_from_settings_with_extension(cppstd):
    """ current cppstd in settings must has GNU extension when extensions is enabled
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", "gnu17", "libstdc++")
    check_min_cppstd(conanfile, cppstd, True)

    conanfile.settings.values["compiler.cppstd"] = "17"
    with pytest.raises(ConanException) as raises:
        check_min_cppstd(conanfile, cppstd, True)
    assert "The cppstd GNU extension is required" == str(raises.value)


@pytest.mark.parametrize("cppstd", ["98", "11", "14", "17"])
def test_valid_min_cppstd_from_settings(cppstd):
    """ valid_min_cppstd must accept cppstd less/equal than cppstd in settings
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", "17", "libstdc++")
    assert valid_min_cppstd(conanfile, cppstd, False)


@pytest.mark.parametrize("cppstd", ["98", "11", "14"])
def test_valid_min_cppstd_from_outdated_settings(cppstd):
    """ valid_min_cppstd returns False when cppstd is greater when supported on settings
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", cppstd, "libstdc++")
    assert not valid_min_cppstd(conanfile, "17", False)


@pytest.mark.parametrize("cppstd", ["98", "11", "14", "17"])
def test_valid_min_cppstd_from_settings_with_extension(cppstd):
    """ valid_min_cppstd must returns True when current cppstd in settings has GNU extension and
        extensions is enabled
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", "gnu17", "libstdc++")
    assert valid_min_cppstd(conanfile, cppstd, True)

    conanfile.settings.values["compiler.cppstd"] = "17"
    assert not valid_min_cppstd(conanfile, cppstd, True)


def test_valid_min_cppstd_unsupported_standard():
    """ valid_min_cppstd must returns False when the compiler does not support a standard
    """
    conanfile = _create_conanfile("gcc", "9", "Linux", None, "libstdc++")
    assert not valid_min_cppstd(conanfile, "42", False)



import platform
import textwrap

import pytest

from conans.test.utils.tools import TestClient

xcode_project = textwrap.dedent("""
    name: app
    targets:
      app:
        type: tool
        platform: macOS
        sources:
          - app
        configFiles:
          Debug: conan_config.xcconfig
          Release: conan_config.xcconfig
    """)

main = textwrap.dedent("""
    #include <iostream>
    #include "hello.h"
    int main(int argc, char *argv[]) {
        hello();
        #ifndef DEBUG
        std::cout << "App Release!" << std::endl;
        #else
        std::cout << "App Debug!" << std::endl;
        #endif
    }
    """)

test = textwrap.dedent("""
    import os
    from conans import ConanFile, tools
    class TestApp(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "VirtualRunEnv"
        def test(self):
            if not tools.cross_building(self):
                self.run("app", env="conanrun")
    """)


@pytest.fixture(scope="module")
def client():
    client = TestClient()
    client.run("new hello/0.1 -m=cmake_lib")
    client.run("create . -s build_type=Release")
    client.run("create . -s build_type=Debug")
    return client


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_xcodebuild
@pytest.mark.tool_xcodegen
def test_project_xcodebuild(client):

    conanfile = textwrap.dedent("""
        import os
        from conans import ConanFile
        from conan.tools.apple import XcodeBuild
        from conan.tools.files import copy
        class MyApplicationConan(ConanFile):
            name = "myapplication"
            version = "1.0"
            requires = "hello/0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "XcodeDeps"
            exports_sources = "app.xcodeproj/*", "app/*"
            def build(self):
                xcode = XcodeBuild(self)
                xcode.build("app.xcodeproj")

            def package(self):
                copy(self, "build/{}/app".format(self.settings.build_type), self.source_folder,
                     os.path.join(self.package_folder, "bin"), keep_path=False)

            def package_info(self):
                self.cpp_info.bindirs = ["bin"]
        """)

    client.save({"conanfile.py": conanfile,
                 "test_package/conanfile.py": test,
                 "app/main.cpp": main,
                 "project.yml": xcode_project}, clean_first=True)
    client.run("install . --build=missing")
    client.run("install . -s build_type=Debug --build=missing")
    client.run_command("xcodegen generate")
    client.run("create . --build=missing")
    assert "hello/0.1: Hello World Release!" in client.out
    assert "App Release!" in client.out
    client.run("create . -s build_type=Debug --build=missing")
    assert "hello/0.1: Hello World Debug!" in client.out
    assert "App Debug!" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_xcodebuild
@pytest.mark.tool_xcodegen
@pytest.mark.skip(reason="Different sdks not installed in CI")
def test_xcodebuild_test_different_sdk(client):

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.apple import XcodeBuild
        class MyApplicationConan(ConanFile):
            name = "myapplication"
            version = "1.0"
            requires = "hello/0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "XcodeDeps"
            exports_sources = "app.xcodeproj/*", "app/*"
            def build(self):
                xcode = XcodeBuild(self)
                xcode.build("app.xcodeproj")
                self.run("otool -l build/Release/app")
        """)

    client.save({"conanfile.py": conanfile,
                 "app/main.cpp": main,
                 "project.yml": xcode_project}, clean_first=True)
    client.run("install . --build=missing")
    client.run("install . -s build_type=Debug --build=missing")
    client.run_command("xcodegen generate")
    client.run("create . --build=missing -s os.sdk=macosx -s os.sdk_version=10.15 "
               "-c tools.apple:sdk_path='/Applications/Xcode11.7.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.15.sdk'")
    assert "sdk 10.15.6" in client.out
    client.run("create . --build=missing -s os.sdk=macosx -s os.sdk_version=11.3 "
               "-c tools.apple:sdk_path='/Applications/Xcode12.5.app/Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX11.3.sdk'")
    assert "sdk 11.3" in client.out


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only for MacOS")
@pytest.mark.tool_xcodebuild
@pytest.mark.tool_xcodegen
def test_missing_sdk(client):

    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conan.tools.apple import XcodeBuild
        class MyApplicationConan(ConanFile):
            name = "myapplication"
            version = "1.0"
            requires = "hello/0.1"
            settings = "os", "compiler", "build_type", "arch"
            generators = "XcodeDeps"
            exports_sources = "app.xcodeproj/*", "app/*"
            def build(self):
                xcode = XcodeBuild(self)
                xcode.build("app.xcodeproj")
        """)

    client.save({"conanfile.py": conanfile,
                 "app/main.cpp": main,
                 "project.yml": xcode_project}, clean_first=True)
    client.run("install . --build=missing")
    client.run("install . -s build_type=Debug --build=missing")
    client.run_command("xcodegen generate")
    client.run("create . --build=missing -s os.sdk=macosx -s os.sdk_version=12.0 "
               "-c tools.apple:sdk_path=notexistingsdk", assert_error=True)



import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.fixture
def client():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Pkg(ConanFile):
            def package_id(self):
                self.info.conf = self.conf
        """)
    client.save({"conanfile.py": conanfile})
    return client


def test_package_id(client):
    profile1 = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Quiet""")
    profile2 = textwrap.dedent("""\
        [conf]
        tools.microsoft.msbuild:verbosity=Minimal""")
    client.save({"profile1": profile1,
                 "profile2": profile2})
    client.run("create . pkg/0.1@ -pr=profile1")
    assert "pkg/0.1:b85ef030da903577bd87d1c92c0524c9c96212b5 - Build" in client.out
    client.run("create . pkg/0.1@ -pr=profile2")
    assert "pkg/0.1:7d2f1590113db99bcd08a4ebd4c841cc0a2e7020 - Build" in client.out
    client.run("create . pkg/0.1@ -c tools.microsoft.msbuild:verbosity=Detailed")
    assert "pkg/0.1:a2e8034244a388f79b31ebec1ddb991bd7b91f48 - Build" in client.out



import os
import unittest
import logging

from conans.client.cache.cache import CONAN_CONF
from conans.client.conf import ConanClientConfigParser
from conans.paths import DEFAULT_PROFILE_NAME
from conans.test.utils.test_files import temp_folder
from conans.util.files import save
from conans.client.tools.oss import environment_append

default_client_conf = '''[storage]
path: ~/.conan/data

[log]
trace_file = "Path/with/quotes"

[general]

'''

default_profile = '''
[settings]
arch=x86_64
build_type=Release
compiler=gcc
compiler.libcxx=libstdc++
compiler.version=4.9
os=Linux

'''


class ClientConfTest(unittest.TestCase):

    def test_quotes(self):
        tmp_dir = temp_folder()
        save(os.path.join(tmp_dir, CONAN_CONF), default_client_conf)
        save(os.path.join(tmp_dir, DEFAULT_PROFILE_NAME), default_profile)
        config = ConanClientConfigParser(os.path.join(tmp_dir, CONAN_CONF))
        self.assertEqual(config.env_vars["CONAN_TRACE_FILE"], "Path/with/quotes")

    def test_proxies(self):
        tmp_dir = temp_folder()
        save(os.path.join(tmp_dir, CONAN_CONF), "")
        config = ConanClientConfigParser(os.path.join(tmp_dir, CONAN_CONF))
        self.assertEqual(None, config.proxies)
        save(os.path.join(tmp_dir, CONAN_CONF), "[proxies]")
        config = ConanClientConfigParser(os.path.join(tmp_dir, CONAN_CONF))
        self.assertNotIn("no_proxy", config.proxies)
        save(os.path.join(tmp_dir, CONAN_CONF), "[proxies]\nno_proxy=localhost")
        config = ConanClientConfigParser(os.path.join(tmp_dir, CONAN_CONF))
        self.assertEqual(config.proxies["no_proxy"], "localhost")


default_client_conf_log = '''[storage]
path: ~/.conan/data

[log]
trace_file = "foo/bar/quotes"
{}

[general]

'''


class ClientConfLogTest(unittest.TestCase):

    def run(self, *args, **kwargs):
        with environment_append({"CONAN_LOGGING_LEVEL": None}):
            super(ClientConfLogTest, self).run(*args, **kwargs)

    def setUp(self):
        self.tmp_dir = temp_folder()
        save(os.path.join(self.tmp_dir, DEFAULT_PROFILE_NAME), default_profile)

    def test_log_level_numbers_critical(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = 50"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.CRITICAL, config.logging_level)

    def test_log_level_numbers_debug(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = 10"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.DEBUG, config.logging_level)

    def test_log_level_numbers_invalid(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = wakawaka"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.CRITICAL, config.logging_level)

    def test_log_level_numbers_env_var_debug(self):
        with environment_append({"CONAN_LOGGING_LEVEL": "10"}):
            save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf)
            config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
            self.assertEqual(logging.DEBUG, config.logging_level)

    def test_log_level_numbers_env_var_debug_text(self):
        with environment_append({"CONAN_LOGGING_LEVEL": "WakaWaka"}):
            save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf)
            config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
            self.assertEqual(logging.CRITICAL, config.logging_level)

    def test_log_level_names_debug(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = debug"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.DEBUG, config.logging_level)

    def test_log_level_names_critical(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = Critical"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.CRITICAL, config.logging_level)

    def test_log_level_names_invalid(self):
        save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf_log.format("level = wakawaka"))
        config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
        self.assertEqual(logging.CRITICAL, config.logging_level)

    def test_log_level_names_env_var_debug(self):
        with environment_append({"CONAN_LOGGING_LEVEL": "Debug"}):
            save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf)
            config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
            self.assertEqual(logging.DEBUG, config.logging_level)

    def test_log_level_names_env_var_warning(self):
        with environment_append({"CONAN_LOGGING_LEVEL": "WARNING"}):
            save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf)
            config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
            self.assertEqual(logging.WARNING, config.logging_level)

    def test_log_level_names_env_var_invalid(self):
        with environment_append({"CONAN_LOGGING_LEVEL": "WakaWaka"}):
            save(os.path.join(self.tmp_dir, CONAN_CONF), default_client_conf)
            config = ConanClientConfigParser(os.path.join(self.tmp_dir, CONAN_CONF))
            self.assertEqual(logging.CRITICAL, config.logging_level)



from conans.test.utils.tools import TestClient, GenConanfile


def test_install():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile("pkga", "0.1").with_package_file("file.h", "0.1")})
    client.run("create . user/channel")

    # Use a consumer with a version range
    client.save({"conanfile.py":
                 GenConanfile("pkgb", "0.1").with_require("pkga/[>=0.1]@user/channel")})
    client.run("create . user/channel")
    client.run("lock create --reference=pkgb/0.1@user/channel --lockfile-out=lock1.lock")

    # We can create a pkga/0.2, but it will not be used
    client.save({"conanfile.py": GenConanfile("pkga", "0.2").with_package_file("file.h", "0.2")})
    client.run("create . user/channel")

    client.run("lock install lock1.lock -g deploy")
    assert "pkga/0.1@user/channel from local cache" in client.out
    file_h = client.load("pkga/file.h")
    assert file_h == "0.1"


def test_install_recipes():
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile("pkga", "0.1").with_package_file("file.h", "0.1")})
    client.run("create . user/channel")

    # Use a consumer with a version range
    client.save({"conanfile.py":
                 GenConanfile("pkgb", "0.1").with_require("pkga/[>=0.1]@user/channel")})
    client.run("create . user/channel")
    client.run("lock create --reference=pkgb/0.1@user/channel --lockfile-out=lock1.lock")

    # We can create a pkga/0.2, but it will not be used
    client.save({"conanfile.py": GenConanfile("pkga", "0.2").with_package_file("file.h", "0.2")})
    client.run("create . user/channel")

    client.run("lock install lock1.lock --recipes")
    assert "pkga/0.1@user/channel:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Cache" in client.out
    assert "pkgb/0.1@user/channel:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Cache" in client.out

    client.run("upload * --all -c")
    client.run("remove * -f")
    client.run("lock install lock1.lock --recipes")

    assert "pkga/0.1@user/channel:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Download" in client.out
    assert "pkgb/0.1@user/channel:cfd10f60aeaa00f5ca1f90b5fe97c3fe19e7ec23 - Download" in client.out




import os

from mock import Mock

from conan.tools.files.files import save_toolchain_args
from conan.tools.gnu import Autotools
from conans import ConanFile
from conans.model.conf import Conf
from conans.test.unittests.util.tools_test import RunnerMock
from conans.test.utils.mocks import MockSettings
from conans.test.utils.test_files import temp_folder


def test_configure_arguments():
    tmp = temp_folder()
    os.chdir(tmp)
    save_toolchain_args({
        "configure_args": "my_configure_args",
        "make_args": "my_make_args"}
    )
    runner = RunnerMock()
    conanfile = ConanFile(Mock(), runner=runner)
    conanfile.settings = MockSettings({})
    conanfile.folders.set_base_install(tmp)
    conanfile.folders.set_base_source(tmp)
    conanfile.conf = Conf()
    conanfile.conf["tools.gnu:make_program"] = "my_make"
    conanfile.conf["tools.build:jobs"] = 23
    ab = Autotools(conanfile)
    ab.configure()
    assert "configure\" my_configure_args" in runner.command_called

    ab = Autotools(conanfile)
    ab.make()
    assert "my_make my_make_args -j23" in runner.command_called




import textwrap

import six

from conans.client import settings_preprocessor
from conans.client.conf import get_default_settings_yml
from conans.errors import ConanV2Exception
from conans.model.settings import Settings
from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class SettingsCppstdTestCase(ConanV2ModeTestCase):
    """ First level setting 'cppstd' is deprecated, Conan will no longer add it to the default settings
        and will fail if used in any recipe or command line
    """

    def test_recipe_invalid(self):
        # If a recipe declares 'settings = "os", ..., "cppstd", it fails
        conanfile = textwrap.dedent("""
            from conans import ConanFile

            class Recipe(ConanFile):
                settings = "os", "cppstd"
        """)
        t = self.get_client()
        t.save({'conanfile.py': conanfile})
        t.run("create . name/version@", assert_error=True)

        self.assertIn("Conan v2 incompatible: Setting 'cppstd' is deprecated", t.out)

    def test_settings_model(self):
        # First level setting 'cppstd' is no longer supported
        settings = Settings.loads(get_default_settings_yml())
        settings.cppstd = "11"
        with six.assertRaisesRegex(self, ConanV2Exception, "Setting 'cppstd' is deprecated"):
                settings_preprocessor.preprocess(settings=settings)


    def test_search(self):
        # First level setting 'cppstd' is no longer supported
        t = self.get_client()
        t.run("info name/version@ -s cppstd=14", assert_error=True)
        self.assertIn("Conan v2 incompatible: Setting 'cppstd' is deprecated", t.out)



import os
import textwrap
import time
from collections import OrderedDict
from time import sleep

from conans.model.ref import ConanFileReference, PackageReference
from conans.paths import CONAN_MANIFEST
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, TestServer, \
    TurboTestClient, GenConanfile
from conans.util.files import load, save


def test_update_binaries():
    client = TestClient(default_server_user=True)
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        from conans.tools import save
        import os, random
        class Pkg(ConanFile):
            def package(self):
                save(os.path.join(self.package_folder, "file.txt"), str(random.random()))
            def deploy(self):
                self.copy("file.txt")
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload Pkg/0.1@lasote/testing --all")

    client2 = TestClient(servers=client.servers, users=client.users)
    client2.run("install Pkg/0.1@lasote/testing")
    value = load(os.path.join(client2.current_folder, "file.txt"))

    time.sleep(1)  # Make sure the new timestamp is later
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload Pkg/0.1@lasote/testing --all")

    client2.run("install Pkg/0.1@lasote/testing")
    new_value = load(os.path.join(client2.current_folder, "file.txt"))
    assert value == new_value

    client2.run("install Pkg/0.1@lasote/testing --update")
    assert "Current package is older than remote upstream one" in client2.out
    new_value = load(os.path.join(client2.current_folder, "file.txt"))
    assert value != new_value

    # Now check newer local modifications are not overwritten
    time.sleep(1)  # Make sure the new timestamp is later
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload Pkg/0.1@lasote/testing --all")

    client2.save({"conanfile.py": conanfile})
    client2.run("create . Pkg/0.1@lasote/testing")
    client2.run("install Pkg/0.1@lasote/testing")
    value2 = load(os.path.join(client2.current_folder, "file.txt"))
    client2.run("install Pkg/0.1@lasote/testing --update")
    assert "Current package is newer than remote upstream one" in client2.out
    new_value = load(os.path.join(client2.current_folder, "file.txt"))
    assert value2 == new_value


def test_update_not_date():
    client = TestClient(default_server_user=True)
    # Regression for https://github.com/conan-io/conan/issues/949
    client.save({"conanfile.py": GenConanfile("Hello0", "1.0")})
    client.run("export . lasote/stable")
    client.save({"conanfile.py": GenConanfile("Hello1", "1.0").
                with_requirement("Hello0/1.0@lasote/stable")},
                clean_first=True)
    client.run("install . --build")
    client.run("upload Hello0/1.0@lasote/stable --all")

    client.run("remote list_ref")
    assert "Hello0/1.0@lasote/stable" in client.out
    client.run("remote list_pref Hello0/1.0@lasote/stable")
    package_reference = "Hello0/1.0@lasote/stable:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9"
    assert package_reference in client.out

    ref = ConanFileReference.loads("Hello0/1.0@lasote/stable")
    pref = PackageReference(ref, "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9")
    export_folder = client.cache.package_layout(ref).export()
    recipe_manifest = os.path.join(export_folder, CONAN_MANIFEST)
    package_folder = client.cache.package_layout(pref.ref).package(pref)
    package_manifest = os.path.join(package_folder, CONAN_MANIFEST)

    def timestamps():
        recipe_timestamp = load(recipe_manifest).splitlines()[0]
        package_timestamp = load(package_manifest).splitlines()[0]
        return recipe_timestamp, package_timestamp

    initial_timestamps = timestamps()

    time.sleep(1)

    # Change and rebuild package
    client.save({"conanfile.py": GenConanfile("Hello0", "1.0").with_test("pass")}, clean_first=True)
    client.run("export . lasote/stable")
    client.run("install Hello0/1.0@lasote/stable --build")

    client.run("remote list_ref")
    assert "Hello0/1.0@lasote/stable" in client.out

    client.run("remote list_pref Hello0/1.0@lasote/stable")
    assert "Hello0/1.0@lasote/stable:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9" in client.out

    rebuild_timestamps = timestamps()
    assert rebuild_timestamps != initial_timestamps

    # back to the consumer, try to update
    client.save({"conanfile.py": GenConanfile("Hello1", "1.0").
                with_requirement("Hello0/1.0@lasote/stable")}, clean_first=True)
    # First assign the preference to a remote, it has been cleared when exported locally
    client.run("install . --update")
    # *1 With revisions here is removing the package because it doesn't belong to the recipe

    assert "Hello0/1.0@lasote/stable from 'default' - Newer" in client.out
    failed_update_timestamps = timestamps()
    assert rebuild_timestamps == failed_update_timestamps

    # hack manifests, put old time
    for manifest_file in (recipe_manifest, package_manifest):
        manifest = load(manifest_file)
        lines = manifest.splitlines()
        lines[0] = "123"
        save(manifest_file, "\n".join(lines))

    client.run("install . --update")
    update_timestamps = timestamps()
    assert update_timestamps == initial_timestamps


def test_reuse():
    client = TestClient(default_server_user=True)
    conanfile = GenConanfile("Hello0", "1.0").with_exports("*").with_package("self.copy('*')")
    client.save({"conanfile.py": conanfile,
                 "header.h": "content1"})
    client.run("export . lasote/stable")
    client.run("install Hello0/1.0@lasote/stable --build")
    client.run("upload Hello0/1.0@lasote/stable --all")

    client2 = TestClient(servers=client.servers, users=client.users)
    client2.run("install Hello0/1.0@lasote/stable")

    assert str(client2.out).count("Downloading conaninfo.txt") == 1

    client.save({"header.h": "//EMPTY!"})
    sleep(1)
    client.run("export . lasote/stable")
    client.run("install Hello0/1.0@lasote/stable --build")
    client.run("upload Hello0/1.0@lasote/stable --all")

    client2.run("install Hello0/1.0@lasote/stable --update")
    ref = ConanFileReference.loads("Hello0/1.0@lasote/stable")
    package_ids = client2.cache.package_layout(ref).package_ids()
    pref = PackageReference(ref, package_ids[0])
    package_path = client2.cache.package_layout(ref).package(pref)
    header = load(os.path.join(package_path, "header.h"))
    assert header == "//EMPTY!"


def test_upload_doesnt_follow_pref():
    servers = OrderedDict()
    servers['r1'] = TestServer()
    servers['r2'] = TestServer()
    client = TestClient(servers=servers, users={"r1": [("lasote", "mypass")],
                                                "r2": [("lasote", "mypass")]})
    ref = "Pkg/0.1@lasote/testing"
    pref = "%s:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9" % ref
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload Pkg/0.1@lasote/testing --all -r r2")
    client.run("remote list_pref Pkg/0.1@lasote/testing")

    assert "%s: r2" % pref in client.out
    client.run("remote remove_ref Pkg/0.1@lasote/testing")

    # It should upload both to r1 (default), not taking into account the pref to r2
    client.run("upload Pkg/0.1@lasote/testing --all")
    assert "Uploading package 1/1: %s to 'r1'" % NO_SETTINGS_PACKAGE_ID in client.out


def test_install_update_following_pref():
    conanfile = textwrap.dedent("""
        import os, random
        from conans import ConanFile, tools
        class Pkg(ConanFile):
            def package(self):
                tools.save(os.path.join(self.package_folder, "file.txt"), str(random.random()))
        """)
    servers = OrderedDict()
    servers["r1"] = TestServer()
    servers["r2"] = TestServer()
    client = TestClient(servers=servers, users={"r1": [("lasote", "mypass")],
                                                "r2": [("lasote", "mypass")]})

    ref = "Pkg/0.1@lasote/testing"
    client.save({"conanfile.py": conanfile})
    client.run("create . %s" % ref)
    client.run("upload %s --all -r r2" % ref)
    client.run("upload %s --all -r r1" % ref)
    # Force recipe to follow r1
    client.run("remote update_ref %s r1" % ref)

    # Update package in r2 from a different client
    time.sleep(1)
    client2 = TestClient(servers=servers, users=client.users)
    ref = "Pkg/0.1@lasote/testing"
    client2.save({"conanfile.py": conanfile})
    client2.run("create . %s" % ref)
    client2.run("upload %s --all -r r2" % ref)

    # Update from client, it will get the binary from r2
    client.run("install %s --update" % ref)
    assert "Pkg/0.1@lasote/testing from 'r1' - Cache" in client.out
    assert "Retrieving package %s from remote 'r2'" % NO_SETTINGS_PACKAGE_ID in client.out


def test_update_binaries_failed():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("install Pkg/0.1@lasote/testing --update")
    assert "Pkg/0.1@lasote/testing: WARN: Can't update, no remote defined" in client.out


def test_update_binaries_no_package_error():
    client = TestClient(default_server_user=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . Pkg/0.1@lasote/testing")
    client.run("upload Pkg/0.1@lasote/testing")
    client.run("remote add_pref Pkg/0.1@lasote/testing:%s default" % NO_SETTINGS_PACKAGE_ID)
    client.run("install Pkg/0.1@lasote/testing --update")
    assert "Pkg/0.1@lasote/testing: WARN: Can't update, no package in remote" in client.out


def test_remove_old_sources():
    # https://github.com/conan-io/conan/issues/1841
    test_server = TestServer()

    def upload(header_content):
        c = TestClient(servers={"default": test_server}, users={"default": [("lasote", "mypass")]})
        base = textwrap.dedent('''
            from conans import ConanFile
            class ConanLib(ConanFile):
                exports_sources = "*"
                def package(self):
                    self.copy("*")
            ''')
        c.save({"conanfile.py": base,
                "header.h": header_content})
        c.run("create . Pkg/0.1@lasote/channel")
        c.run("upload * --confirm --all")
        return c

    client = upload("mycontent1")
    time.sleep(1)
    upload("mycontent2")

    client.run("install Pkg/0.1@lasote/channel -u")

    if client.cache.config.revisions_enabled:
        # The binary package is not updated but downloaded, because the local one we have
        # belongs to a different revision and it is removed
        assert "Pkg/0.1@lasote/channel:%s - Download" % NO_SETTINGS_PACKAGE_ID in client.out
    else:
        assert "Pkg/0.1@lasote/channel:%s - Update" % NO_SETTINGS_PACKAGE_ID in client.out
    assert "Pkg/0.1@lasote/channel: Retrieving package %s" % NO_SETTINGS_PACKAGE_ID in client.out
    ref = ConanFileReference.loads("Pkg/0.1@lasote/channel")
    pref = PackageReference(ref, NO_SETTINGS_PACKAGE_ID)
    header = os.path.join(client.cache.package_layout(pref.ref).package(pref), "header.h")
    assert load(header) == "mycontent2"


def test_fail_usefully_when_failing_retrieving_package():
    ref = ConanFileReference.loads("lib/1.0@conan/stable")
    ref2 = ConanFileReference.loads("lib2/1.0@conan/stable")
    client = TurboTestClient(servers={"default": TestServer()})
    pref1 = client.create(ref)
    client.upload_all(ref)

    client.create(ref2, conanfile=GenConanfile().with_requirement(ref))
    client.upload_all(ref2)

    # remove only the package from pref1
    client.run("remove {} -p {} -f".format(pref1.ref, pref1.id))

    # Now fake the remote url to force a network failure
    client.run("remote update default http://this_not_exist8823.com")
    # Try to install ref2, it will try to download the binary for ref1
    client.run("install {}".format(ref2), assert_error=True)
    assert "ERROR: Error downloading binary package: '{}'".format(pref1) in client.out


def test_evil_insertions():
    ref = ConanFileReference.loads("lib1/1.0@conan/stable")
    ref2 = ConanFileReference.loads("lib2/1.0@conan/stable")

    client = TurboTestClient(servers={"default": TestServer()})
    pref1 = client.create(ref)
    client.upload_all(ref)

    client.create(ref2, conanfile=GenConanfile().with_requirement(ref))
    client.upload_all(ref2)

    client.run("remove {} -p {} -f".format(pref1.ref, pref1.id))

    # Even if we create the package folder artificially, the folder will be discarded and installed again.
    os.makedirs(os.path.join(client.cache_folder, "data", ref.dir_repr(), "package", pref1.id))

    client.run("install {}".format(ref2))

    assert "AssertionError: PREV" not in client.out



import textwrap

from conans.test.utils.tools import TestClient


def test_basic():
    client = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class TestConan(ConanFile):
            name = "hello"
            version = "1.2"
            default_user = "myuser"
            default_channel = "mychannel"
        """)
    client.save({"conanfile.py": conanfile})
    client.run("export .")
    assert "hello/1.2@myuser/mychannel" in client.out
    client.run("search *")
    assert "hello/1.2@myuser/mychannel" in client.out
    client.run("create .")
    assert "hello/1.2@myuser/mychannel" in client.out
    assert "hello/1.2:" not in client.out



import platform

import pytest

from conans.test.utils.conan_v2_tests import ConanV2ModeTestCase


class DefaultConfigTestCase(ConanV2ModeTestCase):

    @pytest.mark.xfail
    def test_package_id_mode(self):
        # TODO: Define package_id_mode for Conan v2
        t = self.get_client()
        self.fail("Define default package_id_mode for Conan v2")
        # self.assertEqual(t.cache.config.default_package_id_mode, "??")
        # t.run('config get general.default_package_id_mode')
        # self.assertEqual(str(t.out).strip(), "semver_direct_mode")

    @pytest.mark.skipif(platform.system() != "Linux", reason="OLD ABI is only detected for Linux/gcc")
    @pytest.mark.tool_gcc
    def test_default_libcxx(self):
        t = self.get_client()
        t.run('profile new --detect autodetected')
        self.assertNotIn("WARNING: GCC OLD ABI COMPATIBILITY", t.out)
        t.run('profile show autodetected')
        self.assertIn("compiler.libcxx=libstdc++11", t.out)



import pytest
from mock import MagicMock

from conan.tools.apple.apple import is_apple_os
from conan.tools.gnu.gnudeps_flags import GnuDepsFlags
from conans.test.utils.mocks import ConanFileMock, MockSettings


@pytest.mark.parametrize("os_", ["Macos", "Windows", "Linux"])
def test_framework_flags_only_for_apple_os(os_):
    """
    Testing GnuDepsFlags attributes exclusively for Apple OS, frameworks and framework_paths
    """
    # Issue: https://github.com/conan-io/conan/issues/10651
    # Issue: https://github.com/conan-io/conan/issues/10640
    settings = MockSettings({"build_type": "Release",
                             "compiler": "gcc",
                             "compiler.version": "10.2",
                             "os": os_,
                             "arch": "x86_64"})
    conanfile = ConanFileMock()
    conanfile.settings = settings
    cpp_info = MagicMock()
    cpp_info.frameworks = ["Foundation"]
    cpp_info.frameworkdirs = ["Framework"]
    gnudepsflags = GnuDepsFlags(conanfile, cpp_info)
    expected_framework = []
    expected_framework_path = []
    if is_apple_os(conanfile):
        expected_framework = ["-framework Foundation"]
        expected_framework_path = ["-F Framework"]
    assert gnudepsflags.frameworks == expected_framework
    assert gnudepsflags.framework_paths == expected_framework_path



import os

import pytest

from conans.client import tools
from conans.model.ref import ConanFileReference
from conans.paths import CONANINFO
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient
from conans.util.files import load


@pytest.fixture()
def client():
    client = TestClient()
    conanfile = GenConanfile("MinGWBuild", "0.1").with_settings("compiler")
    build_msg = """
    def build(self):
        self.output.warn("COMPILER=> %s %s" % (self.name, str(self.settings.compiler)))
    """
    client.save({"conanfile.py": str(conanfile) + build_msg})
    client.run("export . lasote/testing")
    conanfile = GenConanfile("VisualBuild", "0.1").with_requires("MinGWBuild/0.1@lasote/testing")
    conanfile.with_settings("compiler")
    client.save({"conanfile.py": str(conanfile) + build_msg})
    client.run("export . lasote/testing")
    return client


def test_override(client):
    client.run("install VisualBuild/0.1@lasote/testing --build missing -s compiler='Visual Studio' "
               "-s compiler.version=14 -s compiler.runtime=MD "
               "-s MinGWBuild:compiler='gcc' -s MinGWBuild:compiler.libcxx='libstdc++' "
               "-s MinGWBuild:compiler.version=4.8")

    assert "COMPILER=> MinGWBuild gcc" in client.out
    assert "COMPILER=> VisualBuild Visual Studio" in client.out

    # CHECK CONANINFO FILE
    packs_dir = client.cache.package_layout(
        ConanFileReference.loads("MinGWBuild/0.1@lasote/testing")).packages()
    pack_dir = os.path.join(packs_dir, os.listdir(packs_dir)[0])
    conaninfo = load(os.path.join(pack_dir, CONANINFO))
    assert "compiler=gcc" in conaninfo

    # CHECK CONANINFO FILE
    packs_dir = client.cache.package_layout(
        ConanFileReference.loads("VisualBuild/0.1@lasote/testing")).packages()
    pack_dir = os.path.join(packs_dir, os.listdir(packs_dir)[0])
    conaninfo = load(os.path.join(pack_dir, CONANINFO))
    assert "compiler=Visual Studio" in conaninfo
    assert "compiler.version=14" in conaninfo


def test_non_existing_setting(client):
    client.run("install VisualBuild/0.1@lasote/testing --build missing -s compiler='Visual Studio' "
               "-s compiler.version=14 -s compiler.runtime=MD "
               "-s MinGWBuild:missingsetting='gcc' ", assert_error=True)
    assert "settings.missingsetting' doesn't exist" in client.out


def test_override_in_non_existing_recipe(client):
    client.run("install VisualBuild/0.1@lasote/testing --build missing -s compiler='Visual Studio' "
               "-s compiler.version=14 -s compiler.runtime=MD "
               "-s MISSINGID:compiler='gcc' ")

    assert "COMPILER=> MinGWBuild Visual Studio" in client.out
    assert "COMPILER=> VisualBuild Visual Studio" in client.out


def test_override_setting_with_env_variables(client):
    with tools.environment_append({"CONAN_ENV_COMPILER": "Visual Studio",
                                   "CONAN_ENV_COMPILER_VERSION": "14",
                                   "CONAN_ENV_COMPILER_RUNTIME": "MD"}):
        client.run("install VisualBuild/0.1@lasote/testing --build missing")

    assert "COMPILER=> MinGWBuild Visual Studio" in client.out



import textwrap

import six
from parameterized import parameterized

from conans.errors import ConanException
from conans.model.ref import ConanFileReference
from conans.test.integration.graph.core.graph_manager_base import GraphManagerTest
from conans.test.utils.tools import GenConanfile, TestClient


class BuildRequiresGraphTest(GraphManagerTest):

    @parameterized.expand([("recipe", ), ("profile", )])
    def test_basic_build_require(self, build_require):
        # app -(br)-> tool0.1
        tool_ref = ConanFileReference.loads("tool/0.1@user/testing")
        self._cache_recipe(tool_ref, GenConanfile().with_name("tool").with_version("0.1"))
        if build_require == "recipe":
            deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                        .with_build_requires(tool_ref))
        else:
            profile_build_requires = {"*": [ConanFileReference.loads("tool/0.1@user/testing")]}
            deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1"),
                                          profile_build_requires=profile_build_requires)

        # Build requires always apply to the consumer
        self.assertEqual(2, len(deps_graph.nodes))
        app = deps_graph.root
        tool = app.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[], build_deps=[tool], dependents=[],
                         closure=[tool])
        self._check_node(tool, "tool/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[app], closure=[])

    def test_transitive_build_require_recipe(self):
        # app -> lib -(br)-> tool
        tool_ref = ConanFileReference.loads("tool/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(tool_ref, GenConanfile().with_name("tool").with_version("0.1"))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_build_requires(tool_ref))
        deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                    .with_require(lib_ref))

        self.assertEqual(3, len(deps_graph.nodes))
        app = deps_graph.root
        lib = app.dependencies[0].dst
        tool = lib.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], build_deps=[], dependents=[],
                         closure=[lib])
        self._check_node(lib, "lib/0.1@user/testing#123", deps=[], build_deps=[tool],
                         dependents=[app], closure=[tool])
        self._check_node(tool, "tool/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[lib], closure=[])

    def test_loop_build_require(self):
        # app -> lib -(br)-> tool ->|
        #          \<---------------|
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")
        tool_ref = ConanFileReference.loads("tool/0.1@user/testing")

        self._cache_recipe(tool_ref, GenConanfile().with_name("tool").with_version("0.1")
                                                   .with_require(lib_ref))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_build_requires(tool_ref))

        with six.assertRaisesRegex(self, ConanException, "Loop detected in context host:"
                                                         " 'tool/0.1@user/testing' requires"
                                                         " 'lib/0.1@user/testing'"):
            self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                           .with_require(lib_ref))

    def test_transitive_build_require_recipe_profile(self):
        # app -> lib -(br)-> gtest -(br)-> mingw
        # profile \---(br)-> mingw
        # app -(br)-> mingw
        mingw_ref = ConanFileReference.loads("mingw/0.1@user/testing")
        gtest_ref = ConanFileReference.loads("gtest/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(mingw_ref, GenConanfile().with_name("mingw").with_version("0.1"))
        self._cache_recipe(gtest_ref, GenConanfile().with_name("gtest").with_version("0.1"))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_build_requires(gtest_ref))
        profile_build_requires = {"*": [mingw_ref]}
        deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                    .with_require(lib_ref),
                                      profile_build_requires=profile_build_requires)

        self.assertEqual(6, len(deps_graph.nodes))
        app = deps_graph.root
        lib = app.dependencies[0].dst
        gtest = lib.dependencies[0].dst
        mingw_gtest = gtest.dependencies[0].dst
        mingw_lib = lib.dependencies[1].dst
        mingw_app = app.dependencies[1].dst

        self._check_node(app, "app/0.1@", deps=[lib], build_deps=[mingw_app], dependents=[],
                         closure=[mingw_app, lib])

        self._check_node(lib, "lib/0.1@user/testing#123", deps=[], build_deps=[mingw_lib, gtest],
                         dependents=[app], closure=[mingw_lib, gtest])
        self._check_node(gtest, "gtest/0.1@user/testing#123", deps=[], build_deps=[mingw_gtest],
                         dependents=[lib], closure=[mingw_gtest])
        # MinGW leaf nodes
        self._check_node(mingw_gtest, "mingw/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[gtest], closure=[])
        self._check_node(mingw_lib, "mingw/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[lib], closure=[])
        self._check_node(mingw_app, "mingw/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[app], closure=[])

    def test_conflict_transitive_build_requires(self):
        zlib_ref = ConanFileReference.loads("zlib/0.1@user/testing")
        zlib_ref2 = ConanFileReference.loads("zlib/0.2@user/testing")
        gtest_ref = ConanFileReference.loads("gtest/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(zlib_ref, GenConanfile().with_name("zlib").with_version("0.1"))
        self._cache_recipe(zlib_ref2, GenConanfile().with_name("zlib").with_version("0.2"))

        self._cache_recipe(gtest_ref, GenConanfile().with_name("gtest").with_version("0.1")
                                                    .with_require(zlib_ref2))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_require(zlib_ref)
                                                  .with_build_requires(gtest_ref))

        with six.assertRaisesRegex(self, ConanException,
                                   "Conflict in gtest/0.1@user/testing:\n"
                                   "    'gtest/0.1@user/testing' requires 'zlib/0.2@user/testing' "
                                   "while 'lib/0.1@user/testing' requires 'zlib/0.1@user/testing'."
                                   "\n    To fix this conflict you need to override the package "
                                   "'zlib' in your root package."):
            self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                           .with_require(lib_ref))

    def test_not_conflict_transitive_build_requires(self):
        # Same as above, but gtest->(build_require)->zlib2
        zlib_ref = ConanFileReference.loads("zlib/0.1@user/testing")
        zlib_ref2 = ConanFileReference.loads("zlib/0.2@user/testing")
        gtest_ref = ConanFileReference.loads("gtest/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(zlib_ref, GenConanfile().with_name("zlib").with_version("0.1"))
        self._cache_recipe(zlib_ref2, GenConanfile().with_name("zlib").with_version("0.2"))

        self._cache_recipe(gtest_ref, GenConanfile().with_name("gtest").with_version("0.1")
                                                    .with_build_requires(zlib_ref2))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_require(zlib_ref)
                                                  .with_build_requires(gtest_ref))

        graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                               .with_require(lib_ref))

        self.assertEqual(5, len(graph.nodes))
        app = graph.root
        lib = app.dependencies[0].dst
        zlib = lib.dependencies[0].dst
        gtest = lib.dependencies[1].dst
        zlib2 = gtest.dependencies[0].dst
        self._check_node(app, "app/0.1@", deps=[lib], build_deps=[], dependents=[],
                         closure=[lib, zlib])

        self._check_node(lib, "lib/0.1@user/testing#123", deps=[zlib], build_deps=[gtest],
                         dependents=[app], closure=[gtest, zlib])

        self._check_node(gtest, "gtest/0.1@user/testing#123", deps=[], build_deps=[zlib2],
                         dependents=[lib], closure=[zlib2])

    def test_diamond_no_option_conflict_build_requires(self):
        # Same as above, but gtest->(build_require)->zlib2
        zlib_ref = ConanFileReference.loads("zlib/0.1@user/testing")
        gtest_ref = ConanFileReference.loads("gtest/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(zlib_ref, GenConanfile().with_name("zlib").with_version("0.1")
                                                   .with_option("shared", [True, False])
                                                   .with_default_option("shared", False))

        self._cache_recipe(gtest_ref, GenConanfile().with_name("gtest").with_version("0.1")
                                                    .with_require(zlib_ref)
                                                    .with_default_option("zlib:shared", True))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_require(zlib_ref)
                                                  .with_build_requires(gtest_ref))

        graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                               .with_require(lib_ref))

        self.assertEqual(4, len(graph.nodes))
        app = graph.root
        lib = app.dependencies[0].dst
        zlib = lib.dependencies[0].dst
        self.assertFalse(bool(zlib.conanfile.options.shared))
        gtest = lib.dependencies[1].dst
        zlib2 = gtest.dependencies[0].dst
        self.assertIs(zlib, zlib2)
        self._check_node(app, "app/0.1@", deps=[lib], build_deps=[], dependents=[],
                         closure=[lib, zlib])

        self._check_node(lib, "lib/0.1@user/testing#123", deps=[zlib], build_deps=[gtest],
                         dependents=[app], closure=[gtest, zlib])

        self._check_node(gtest, "gtest/0.1@user/testing#123", deps=[zlib2], build_deps=[],
                         dependents=[lib], closure=[zlib2])

    def test_diamond_option_conflict_build_requires(self):
        # Same as above, but gtest->(build_require)->zlib2
        zlib_ref = ConanFileReference.loads("zlib/0.1@user/testing")
        gtest_ref = ConanFileReference.loads("gtest/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(zlib_ref, GenConanfile().with_name("zlib").with_version("0.1")
                                                   .with_option("shared", [True, False])
                                                   .with_default_option("shared", False))
        configure = """
    def configure(self):
        self.options["zlib"].shared=True
        """
        gtest = str(GenConanfile().with_name("gtest").with_version("0.1")
                                  .with_require(zlib_ref)) + configure
        self._cache_recipe(gtest_ref, gtest)
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_require(zlib_ref)
                                                  .with_build_requires(gtest_ref))

        with six.assertRaisesRegex(self, ConanException,
                                   "tried to change zlib/0.1@user/testing option shared to True"):
            self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                           .with_require(lib_ref))

    def test_consecutive_diamonds_build_requires(self):
        # app -> libe0.1 -------------> libd0.1 -> libb0.1 -------------> liba0.1
        #    \-(build-require)-> libf0.1 ->/    \-(build-require)->libc0.1 ->/
        liba_ref = ConanFileReference.loads("liba/0.1@user/testing")
        libb_ref = ConanFileReference.loads("libb/0.1@user/testing")
        libc_ref = ConanFileReference.loads("libc/0.1@user/testing")
        libd_ref = ConanFileReference.loads("libd/0.1@user/testing")
        libe_ref = ConanFileReference.loads("libe/0.1@user/testing")
        libf_ref = ConanFileReference.loads("libf/0.1@user/testing")
        self._cache_recipe(liba_ref, GenConanfile().with_name("liba").with_version("0.1"))
        self._cache_recipe(libb_ref, GenConanfile().with_name("libb").with_version("0.1")
                                                   .with_require(liba_ref))
        self._cache_recipe(libc_ref, GenConanfile().with_name("libc").with_version("0.1")
                                                   .with_require(liba_ref))
        self._cache_recipe(libd_ref, GenConanfile().with_name("libd").with_version("0.1")
                                                   .with_require(libb_ref)
                                                   .with_build_requires(libc_ref))
        self._cache_recipe(libe_ref, GenConanfile().with_name("libe").with_version("0.1")
                                                   .with_require(libd_ref))
        self._cache_recipe(libf_ref, GenConanfile().with_name("libf").with_version("0.1")
                                                   .with_require(libd_ref))
        deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                    .with_require(libe_ref)
                                                    .with_build_requires(libf_ref))

        self.assertEqual(7, len(deps_graph.nodes))
        app = deps_graph.root
        libe = app.dependencies[0].dst
        libf = app.dependencies[1].dst
        libd = libe.dependencies[0].dst
        libb = libd.dependencies[0].dst
        libc = libd.dependencies[1].dst
        liba = libc.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[libe], build_deps=[libf], dependents=[],
                         closure=[libf, libe, libd, libb, liba])
        self._check_node(libe, "libe/0.1@user/testing#123", deps=[libd], build_deps=[],
                         dependents=[app], closure=[libd, libb, liba])
        self._check_node(libf, "libf/0.1@user/testing#123", deps=[libd], build_deps=[],
                         dependents=[app], closure=[libd, libb, liba])
        self._check_node(libd, "libd/0.1@user/testing#123", deps=[libb], build_deps=[libc],
                         dependents=[libe, libf], closure=[libc, libb, liba])
        self._check_node(libc, "libc/0.1@user/testing#123", deps=[liba], build_deps=[],
                         dependents=[libd], closure=[liba])
        self._check_node(libb, "libb/0.1@user/testing#123", deps=[liba], build_deps=[],
                         dependents=[libd], closure=[liba])
        self._check_node(liba, "liba/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[libb, libc], closure=[])

    def test_parallel_diamond_build_requires(self):
        # app --------> libb0.1 ---------> liba0.1
        #    \--------> libc0.1 ----------->/
        #    \-(build_require)-> libd0.1 ->/
        liba_ref = ConanFileReference.loads("liba/0.1@user/testing")
        libb_ref = ConanFileReference.loads("libb/0.1@user/testing")
        libc_ref = ConanFileReference.loads("libc/0.1@user/testing")
        libd_ref = ConanFileReference.loads("libd/0.1@user/testing")
        self._cache_recipe(liba_ref, GenConanfile().with_name("liba").with_version("0.1"))
        self._cache_recipe(libb_ref, GenConanfile().with_name("libb").with_version("0.1")
                                                   .with_require(liba_ref))
        self._cache_recipe(libc_ref, GenConanfile().with_name("libc").with_version("0.1")
                                                   .with_require(liba_ref))
        self._cache_recipe(libd_ref, GenConanfile().with_name("libd").with_version("0.1")
                                                   .with_require(liba_ref))
        deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                    .with_require(libb_ref)
                                                    .with_require(libc_ref)
                                                    .with_build_requires(libd_ref))

        self.assertEqual(5, len(deps_graph.nodes))
        app = deps_graph.root
        libb = app.dependencies[0].dst
        libc = app.dependencies[1].dst
        libd = app.dependencies[2].dst
        liba = libb.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[libb, libc], build_deps=[libd],
                         dependents=[], closure=[libd, libb, libc, liba])
        self._check_node(libb, "libb/0.1@user/testing#123", deps=[liba], build_deps=[],
                         dependents=[app], closure=[liba])
        self._check_node(libc, "libc/0.1@user/testing#123", deps=[liba], build_deps=[],
                         dependents=[app], closure=[liba])
        self._check_node(libd, "libd/0.1@user/testing#123", deps=[liba], build_deps=[],
                         dependents=[app], closure=[liba])
        self._check_node(liba, "liba/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[libb, libc, libd], closure=[])

    def test_build_require_private(self):
        # app -> lib -(br)-> tool -(private)-> zlib
        zlib_ref = ConanFileReference.loads("zlib/0.1@user/testing")
        tool_ref = ConanFileReference.loads("tool/0.1@user/testing")
        lib_ref = ConanFileReference.loads("lib/0.1@user/testing")

        self._cache_recipe(zlib_ref, GenConanfile().with_name("zlib").with_version("0.1"))
        self._cache_recipe(tool_ref, GenConanfile().with_name("tool").with_version("0.1")
                                                   .with_require(zlib_ref, private=True))
        self._cache_recipe(lib_ref, GenConanfile().with_name("lib").with_version("0.1")
                                                  .with_build_requires(tool_ref))
        deps_graph = self.build_graph(GenConanfile().with_name("app").with_version("0.1")
                                                    .with_require(lib_ref))

        self.assertEqual(4, len(deps_graph.nodes))
        app = deps_graph.root
        lib = app.dependencies[0].dst
        tool = lib.dependencies[0].dst
        zlib = tool.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], build_deps=[], dependents=[],
                         closure=[lib])

        self._check_node(lib, "lib/0.1@user/testing#123", deps=[], build_deps=[tool],
                         dependents=[app], closure=[tool])

        self._check_node(tool, "tool/0.1@user/testing#123", deps=[zlib], build_deps=[],
                         dependents=[lib], closure=[zlib])

        self._check_node(zlib, "zlib/0.1@user/testing#123", deps=[], build_deps=[],
                         dependents=[tool], closure=[])

    def test_build_require_conflict(self):
        # https://github.com/conan-io/conan/issues/4931
        # cheetah -> gazelle -> grass
        #    \--(br)------0.2----/
        grass01_ref = ConanFileReference.loads("grass/0.1@user/testing")
        grass02_ref = ConanFileReference.loads("grass/0.2@user/testing")
        gazelle_ref = ConanFileReference.loads("gazelle/0.1@user/testing")

        self._cache_recipe(grass01_ref, GenConanfile().with_name("grass").with_version("0.1"))
        self._cache_recipe(grass02_ref, GenConanfile().with_name("grass").with_version("0.2"))
        self._cache_recipe(gazelle_ref, GenConanfile().with_name("gazelle").with_version("0.1")
                                                      .with_require(grass01_ref))

        with six.assertRaisesRegex(self, ConanException,
                                   "Conflict in cheetah/0.1:\n"
                                   "    'cheetah/0.1' requires 'grass/0.2@user/testing' while "
                                   "'gazelle/0.1@user/testing' requires 'grass/0.1@user/testing'.\n"
                                   "    To fix this conflict you need to override the package "
                                   "'grass' in your root package."):
            self.build_graph(GenConanfile().with_name("cheetah").with_version("0.1")
                                           .with_require(gazelle_ref)
                                           .with_build_requires(grass02_ref))

    def test_build_require_link_order(self):
        # https://github.com/conan-io/conan/issues/4931
        # cheetah -> gazelle -> grass
        #    \--(br)------------/
        grass01_ref = ConanFileReference.loads("grass/0.1@user/testing")
        gazelle_ref = ConanFileReference.loads("gazelle/0.1@user/testing")

        self._cache_recipe(grass01_ref, GenConanfile().with_name("grass").with_version("0.1"))
        self._cache_recipe(gazelle_ref, GenConanfile().with_name("gazelle").with_version("0.1")
                                                      .with_require(grass01_ref))

        deps_graph = self.build_graph(GenConanfile().with_name("cheetah").with_version("0.1")
                                                    .with_require(gazelle_ref)
                                                    .with_build_requires(grass01_ref))

        self.assertEqual(3, len(deps_graph.nodes))
        cheetah = deps_graph.root
        gazelle = cheetah.dependencies[0].dst
        grass = gazelle.dependencies[0].dst
        grass2 = cheetah.dependencies[1].dst

        self._check_node(cheetah, "cheetah/0.1@", deps=[gazelle], build_deps=[grass2],
                         dependents=[], closure=[gazelle, grass])
        self.assertListEqual(list(cheetah.conanfile.deps_cpp_info.libs),
                             ['mylibgazelle0.1lib', 'mylibgrass0.1lib'])

    def test_test_require(self):
        # app -(tr)-> gtest/0.1
        # This test should survive in 2.0
        tool_ref = ConanFileReference.loads("gtest/0.1")
        self._cache_recipe(tool_ref, GenConanfile("gtest", "0.1"))

        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Pkg(ConanFile):
                name = "app"
                version = "0.1"
                def build_requirements(self):
                    self.test_requires("gtest/0.1")
            """)
        deps_graph = self.build_graph(conanfile)

        # Build requires always apply to the consumer
        self.assertEqual(2, len(deps_graph.nodes))
        app = deps_graph.root
        tool = app.dependencies[0].dst

        self._check_node(app, "app/0.1@", deps=[], build_deps=[tool], dependents=[],
                         closure=[tool])
        self._check_node(tool, "gtest/0.1#123", deps=[], build_deps=[],
                         dependents=[app], closure=[])


def test_tool_requires():
    """Testing temporary tool_requires attribute being "an alias" of build_require and
    introduced to provide a compatible recipe with 2.0. At 2.0, the meaning of a build require being
    a 'tool' will be a tool_require."""

    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . tool1/1.0@")
    client.run("create . tool2/1.0@")
    client.run("create . tool3/1.0@")
    client.run("create . tool4/1.0@")

    consumer = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            tool_requires = "tool2/1.0"
            build_requires = "tool3/1.0"

            def build_requirements(self):
                self.tool_requires("tool1/1.0")
                self.build_requires("tool4/1.0")

            def generate(self):
                assert len(self.dependencies.build.values()) == 4
    """)
    client.save({"conanfile.py": consumer})
    client.run("create . consumer/1.0@")
    assert """Build requirements
    tool1/1.0 from local cache - Cache
    tool2/1.0 from local cache - Cache
    tool3/1.0 from local cache - Cache
    tool4/1.0 from local cache - Cache""" in client.out



import json
import os
import pytest

from conans.errors import ConanException
from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient
from conans.util.files import load, save_append
from conans.test.utils.test_files import temp_folder
from conans.client.tools import environment_append
from conans.model.conf import BUILT_IN_CONFS


def _assert_dict_subset(expected, actual):
    actual = {k: v for k, v in actual.items() if k in expected}
    assert all(v == actual[k] for k, v in expected.items()) and len(expected) == len(actual)


def test_basic():
    """ config get MUST show full file
    """
    client = TestClient()
    client.run("config get")
    assert "default_profile = default" in client.out
    assert "path = ./data" in client.out


def test_storage():
    """ config get storage.path MUST show cache path
    """
    client = TestClient()
    client.run("config get storage")
    assert "path = ./data" in client.out

    client.run("config get storage.path")
    full_path = os.path.join(client.cache_folder, "data")
    assert full_path in client.out
    assert "path:" not in client.out


def test_errors():
    """ Invalid properties MUST be considered an error
    """
    client = TestClient()
    client.run("config get whatever", assert_error=True)
    assert "'whatever' is not a section of conan.conf" in client.out

    client.run("config get whatever.what", assert_error=True)
    assert "'whatever' is not a section of conan.conf" in client.out

    client.run("config get storage.what", assert_error=True)
    assert "'what' doesn't exist in [storage]" in client.out

    client.run('config set proxies=https:', assert_error=True)
    assert "You can't set a full section, please specify a section.key=value" in client.out

    client.run('config set proxies.http:Value', assert_error=True)
    assert "Please specify 'key=value'" in client.out


def test_define():
    """ config set MUST add/override conan.conf properties
    """
    client = TestClient()
    client.run("config set general.fakeos=Linux")
    conf_file = load(client.cache.conan_conf_path)
    assert "fakeos = Linux" in conf_file

    client.run('config set general.compiler="Other compiler"')
    conf_file = load(client.cache.conan_conf_path)
    assert 'compiler = Other compiler' in conf_file

    client.run('config set general.compiler.version=123.4.5')
    conf_file = load(client.cache.conan_conf_path)
    assert 'compiler.version = 123.4.5' in conf_file
    assert "14" not in conf_file

    client.run('config set general.new_setting=mysetting')
    conf_file = load(client.cache.conan_conf_path)
    assert 'new_setting = mysetting' in conf_file

    client.run('config set proxies.https=myurl')
    conf_file = load(client.cache.conan_conf_path)
    assert "https = myurl" in conf_file.splitlines()


def test_set_with_weird_path():
    """ config MUST support symbols in path
       https://github.com/conan-io/conan/issues/4110
    """
    client = TestClient()
    client.run("config set log.trace_file=/recipe-release%2F0.6.1")
    client.run("config get log.trace_file")
    assert "/recipe-release%2F0.6.1", client.out


def test_remove():
    """ config rm MUST remove property value and key
    """
    client = TestClient()
    client.run('config set proxies.https=myurl')
    client.run('config rm proxies.https')
    conf_file = load(client.cache.conan_conf_path)
    assert 'myurl' not in conf_file


def test_remove_section():
    """ config rm MUST remove sections
    """
    client = TestClient()
    client.run('config rm proxies')
    conf_file = load(client.cache.conan_conf_path)
    assert '[proxies]' not in conf_file


def test_remove_envvar():
    """ config MUST add/remove env vars in conan.conf
    """
    client = TestClient()
    client.run('config set env.MY_VAR=MY_VALUE')
    conf_file = load(client.cache.conan_conf_path)
    assert 'MY_VAR = MY_VALUE' in conf_file
    client.run('config rm env.MY_VAR')
    conf_file = load(client.cache.conan_conf_path)
    assert 'MY_VAR' not in conf_file


def test_missing_subarguments():
    """ config MUST run  with a subcommand. Otherwise, it MUST exits with error.
    """
    client = TestClient()
    client.run("config", assert_error=True)
    assert "ERROR: Exiting with code: 2" in client.out


def test_config_home_default():
    """ config home MUST show conan home path
    """
    client = TestClient()
    client.run("config home")
    assert client.cache.cache_folder in client.out
    client.run("config home --json home.json")
    _assert_dict_subset({"home": client.cache.cache_folder},json.loads(client.load("home.json")))


def test_config_home_custom_home_dir():
    """ config home MUST accept CONAN_USER_HOME as custom home path
    """
    cache_folder = os.path.join(temp_folder(), "custom")
    with environment_append({"CONAN_USER_HOME": cache_folder}):
        client = TestClient(cache_folder=cache_folder)
        client.run("config home")
        assert cache_folder in client.out
        client.run("config home --json home.json")
        _assert_dict_subset({"home": cache_folder}, json.loads(client.load("home.json")))


def test_config_home_custom_install():
    """ config install MUST accept CONAN_USER_HOME as custom home path
    """
    cache_folder = os.path.join(temp_folder(), "custom")
    with environment_append({"CONAN_USER_HOME": cache_folder}):
        client = TestClient(cache_folder=cache_folder, cache_autopopulate=False)
        client.save({"conanfile.py": GenConanfile()})
        client.run("install .")
        assert "conanfile.py: Installing package" in client.out


def test_config_home_short_home_dir():
    """ conan home MUST no be a conan cache sub folder
    """
    cache_folder = os.path.join(temp_folder(), "custom")
    with environment_append({"CONAN_USER_HOME_SHORT": cache_folder}):
        with pytest.raises(ConanException) as excinfo:
            TestClient(cache_folder=cache_folder)
            assert "cannot be a subdirectory of the conan cache" in str(excinfo.value)


def test_config_home_short_home_dir_contains_cache_dir():
    """ short path property for home MUST be equals to CONAN_USER_HOME_SHORT
        https://github.com/conan-io/conan/issues/6273
    """
    cache_folder = os.path.join(temp_folder(), "custom")
    short_path_home_folder = cache_folder + '_short'
    with environment_append({"CONAN_USER_HOME_SHORT": short_path_home_folder}):
        client = TestClient(cache_folder=cache_folder)
        assert client.cache.config.short_paths_home == short_path_home_folder


def test_config_user_home_short_path():
    """ When general.user_home_short is configured, short_paths MUST obey its path
    """
    short_folder = os.path.join(temp_folder(), "short").replace("\\", "/")
    with environment_append({"CONAN_USER_HOME_SHORT": ""}):
        client = TestClient()
        client.run("config set general.user_home_short='{}'".format(short_folder))
        client.save({"conanfile.py": GenConanfile().with_short_paths(True)})
        client.run("create . foobar/0.1.0@user/testing")
        assert client.cache.config.short_paths_home == short_folder


def test_config_user_home_short_none():
    """ When general.user_home_short is None, short_paths MUST use cache folder
    """
    with environment_append({"CONAN_USER_HOME_SHORT": ""}):
        client = TestClient()
        client.run('config set general.user_home_short=None')
        client.save({"conanfile.py": GenConanfile().with_short_paths(True)})
        client.run("create . foobar/0.1.0@user/testing")
        assert client.cache.config.short_paths_home == "None"


def test_init():
    """ config init MUST initialize conan.conf, remotes, settings and default profile
    """
    client = TestClient()
    client.run('config init')
    assert os.path.exists(client.cache.conan_conf_path)
    assert os.path.exists(client.cache.remotes_path)
    assert os.path.exists(client.cache.settings_path)
    assert os.path.exists(client.cache.default_profile_path)


def test_init_overwrite():
    """ config init --force MUST override current content
    """
    client = TestClient()
    client.run('config init')
    dummy_content = 'DUMMY CONTENT. SHOULD BE REMOVED!'
    save_append(client.cache.conan_conf_path, dummy_content)
    save_append(client.cache.remotes_path, dummy_content)
    save_append(client.cache.settings_path, dummy_content)
    save_append(client.cache.default_profile_path, dummy_content)

    client.run('config init --force')
    assert dummy_content not in load(client.cache.conan_conf_path)
    assert dummy_content not in load(client.cache.conan_conf_path)
    assert dummy_content not in load(client.cache.settings_path)
    assert dummy_content not in load(client.cache.remotes_path)
    assert dummy_content not in load(client.cache.default_profile_path)


def test_config_list():
    """ config list MUST show all configuration available for global.conf
    """
    client = TestClient()
    client.run('config list')
    assert "Supported Conan *experimental* global.conf and [conf] properties:" in client.out
    for key, description in BUILT_IN_CONFS.items():
        assert "{}: {}".format(key, description) in client.out



import os
import platform
import unittest

import mock
import pytest
import six
from mock.mock import patch
from six import StringIO

from conans import Settings
from conans.client import tools
from conans.client.conf import get_default_settings_yml
from conans.client.output import ConanOutput
from conans.client.runner import ConanRunner
from conans.client.tools.win import vcvars_dict
from conans.errors import ConanException
from conans.test.utils.tools import TestClient
from conans.test.utils.mocks import TestBufferConanOutput


class VCVarsTest(unittest.TestCase):
    def setUp(self):
        self.output = TestBufferConanOutput()

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_echo(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        cmd = tools.vcvars_command(settings, output=self.output)
        output = TestBufferConanOutput()
        runner = ConanRunner(print_commands_to_output=True, output=output)
        runner(cmd + " && set vs140comntools")
        self.assertIn("vcvarsall.bat", str(output))
        self.assertIn("VS140COMNTOOLS=", str(output))
        with tools.environment_append({"VisualStudioVersion": "14"}):
            output = TestBufferConanOutput()
            runner = ConanRunner(print_commands_to_output=True, output=output)
            cmd = tools.vcvars_command(settings, output=self.output)
            runner(cmd + " && set vs140comntools")
            self.assertNotIn("vcvarsall.bat", str(output))
            self.assertIn("Conan:vcvars already set", str(output))
            self.assertIn("VS140COMNTOOLS=", str(output))

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_with_store_echo(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "WindowsStore"
        settings.os.version = "8.1"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        cmd = tools.vcvars_command(settings, output=self.output)
        self.assertIn("store 8.1", cmd)
        with tools.environment_append({"VisualStudioVersion": "14"}):
            cmd = tools.vcvars_command(settings, output=self.output)
            self.assertEqual("echo Conan:vcvars already set", cmd)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_env_not_duplicated_path(self):
        """vcvars is not looking at the current values of the env vars, with PATH it is a problem
        because you can already have set some of the vars and accumulate unnecessary entries."""
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.arch_build = "x86_64"

        # Set the env with a PATH containing the vcvars paths
        tmp = tools.vcvars_dict(settings, only_diff=False, output=self.output)
        tmp = {key.lower(): value for key, value in tmp.items()}
        with tools.environment_append({"path": tmp["path"]}):
            previous_path = os.environ["PATH"].split(";")
            # Duplicate the path, inside the tools.vcvars shouldn't have repeated entries in PATH
            with tools.vcvars(settings, output=self.output):
                path = os.environ["PATH"].split(";")
                values_count = {value: path.count(value) for value in path}
                for value, counter in values_count.items():
                    if value and (counter > 1) and previous_path.count(value) != counter:
                        # If the entry was already repeated before calling "tools.vcvars" we keep it
                        self.fail("The key '%s' has been repeated" % value)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_filter_known_paths(self):
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.arch_build = "x86_64"
        with tools.environment_append({"PATH": ["custom_path", "WindowsFake"]}):
            tmp = tools.vcvars_dict(settings, only_diff=False,
                                    filter_known_paths=True, output=self.output)
            with tools.environment_append(tmp):
                self.assertNotIn("custom_path", os.environ["PATH"])
                self.assertIn("WindowsFake",  os.environ["PATH"])
            tmp = tools.vcvars_dict(settings, only_diff=False,
                                    filter_known_paths=False, output=self.output)
            with tools.environment_append(tmp):
                self.assertIn("custom_path", os.environ["PATH"])
                self.assertIn("WindowsFake", os.environ["PATH"])

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_amd64_32_cross_building_support(self):
        # amd64_x86 crossbuilder
        settings = Settings.loads(get_default_settings_yml())
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "15"
        settings.arch = "x86"
        settings.arch_build = "x86_64"
        cmd = tools.vcvars_command(settings, output=self.output)
        self.assertIn('vcvarsall.bat" amd64_x86', cmd)

        # It follows arch_build first
        settings.arch_build = "x86"
        cmd = tools.vcvars_command(settings, output=self.output)
        self.assertIn('vcvarsall.bat" x86', cmd)

    def test_vcvars_raises_when_not_found(self):
        text = """
os: [Windows]
arch: [x86_64]
arch_build: [x86_64]
compiler:
    Visual Studio:
        version: ["5"]
        """
        settings = Settings.loads(text)
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "5"
        settings.arch = "x86_64"
        settings.arch_build = "x86_64"
        with six.assertRaisesRegex(self, ConanException,
                                   "VS non-existing installation: Visual Studio 5"):
            output = ConanOutput(StringIO())
            tools.vcvars_command(settings, output=output)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_constrained(self):
        new_out = StringIO()
        output = ConanOutput(new_out)

        text = """os: [Windows]
compiler:
    Visual Studio:
        version: ["14"]
        """
        settings = Settings.loads(text)
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        with six.assertRaisesRegex(self, ConanException,
                                   "compiler.version setting required for vcvars not defined"):
            tools.vcvars_command(settings, output=output)

        new_out = StringIO()
        output = ConanOutput(new_out)
        settings.compiler.version = "14"
        with tools.environment_append({"vs140comntools": "path/to/fake"}):
            tools.vcvars_command(settings, output=output)
            with tools.environment_append({"VisualStudioVersion": "12"}):
                with six.assertRaisesRegex(self, ConanException,
                                           "Error, Visual environment already set to 12"):
                    tools.vcvars_command(settings, output=output)

            with tools.environment_append({"VisualStudioVersion": "12"}):
                # Not raising
                tools.vcvars_command(settings, force=True, output=output)

    def test_vcvars_context_manager(self):
        conanfile = """
from conans import ConanFile, tools

class MyConan(ConanFile):
    name = "MyConan"
    version = "0.1"
    settings = "os", "compiler"

    def build(self):
        with tools.vcvars(self.settings, only_diff=True):
            self.output.info("VCINSTALLDIR set to: " + str(tools.get_env("VCINSTALLDIR")))
"""
        client = TestClient()
        client.save({"conanfile.py": conanfile})

        if platform.system() == "Windows":
            client.run("create . conan/testing")
            self.assertNotIn("VCINSTALLDIR set to: None", client.out)
        else:
            client.run("create . conan/testing")
            self.assertIn("VCINSTALLDIR set to: None", client.out)

    @pytest.mark.tool_visual_studio
    @pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
    def test_vcvars_dict_diff(self):
        text = """
os: [Windows]
compiler:
    Visual Studio:
        version: ["14"]
        """
        settings = Settings.loads(text)
        settings.os = "Windows"
        settings.compiler = "Visual Studio"
        settings.compiler.version = "14"
        with tools.environment_append({"MYVAR": "1"}):
            ret = vcvars_dict(settings, only_diff=False, output=self.output)
            self.assertIn("MYVAR", ret)
            self.assertIn("VCINSTALLDIR", ret)

            ret = vcvars_dict(settings, output=self.output)
            self.assertNotIn("MYVAR", ret)
            self.assertIn("VCINSTALLDIR", ret)

        my_lib_paths = "C:\\PATH\\TO\\MYLIBS;C:\\OTHER_LIBPATH"
        with tools.environment_append({"LIBPATH": my_lib_paths}):
            ret = vcvars_dict(settings, only_diff=False, output=self.output)
            str_var_value = os.pathsep.join(ret["LIBPATH"])
            self.assertTrue(str_var_value.endswith(my_lib_paths))

            # Now only a diff, it should return the values as a list, but without the old values
            ret = vcvars_dict(settings, only_diff=True, output=self.output)
            self.assertEqual(ret["LIBPATH"], str_var_value.split(os.pathsep)[0:-2])

            # But if we apply both environments, they are composed correctly
            with tools.environment_append(ret):
                self.assertEqual(os.environ["LIBPATH"], str_var_value)

    def test_vcvars_dict(self):
        # https://github.com/conan-io/conan/issues/2904
        output_with_newline_and_spaces = """
     PROCESSOR_ARCHITECTURE=AMD64

PROCESSOR_IDENTIFIER=Intel64 Family 6 Model 158 Stepping 9, GenuineIntel


 PROCESSOR_LEVEL=6

PROCESSOR_REVISION=9e09


set nl=^
env_var=
without_equals_sign

ProgramFiles(x86)=C:\Program Files (x86)

"""

        def vcvars_command_mock(*args, **kwargs):
            return "unused command"

        def subprocess_check_output_mock(cmd):
            self.assertIn("unused command", cmd)
            return output_with_newline_and_spaces

        with mock.patch('conans.client.tools.win.vcvars_command', new=vcvars_command_mock):
            with patch('conans.client.tools.win.check_output_runner',
                       new=subprocess_check_output_mock):
                vcvars = tools.vcvars_dict(None, only_diff=False, output=self.output)
                self.assertEqual(vcvars["PROCESSOR_ARCHITECTURE"], "AMD64")
                self.assertEqual(vcvars["PROCESSOR_IDENTIFIER"],
                                 "Intel64 Family 6 Model 158 Stepping 9, GenuineIntel")
                self.assertEqual(vcvars["PROCESSOR_LEVEL"], "6")
                self.assertEqual(vcvars["PROCESSOR_REVISION"], "9e09")
                self.assertEqual(vcvars["ProgramFiles(x86)"], "C:\\Program Files (x86)")
